"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.stage._generated.models.credentials import Credentials, CredentialsModel
from snowflake.core.stage._generated.models.stage_directory_table import StageDirectoryTable, StageDirectoryTableModel
from snowflake.core.stage._generated.models.stage_encryption import StageEncryption, StageEncryptionModel


class Stage(BaseModel):
    """A model object representing the Stage resource.

    Constructs an object of type Stage with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    kind : str,  default 'PERMANENT'
        Specifies whether the stage is permanent or temporary.
    url : str, optional
        URL for the external stage; blank for an internal stage.
    endpoint : str, optional
        The S3-compatible API endpoint associated with the stage; always NULL for stages that are not S3-compatible.
    storage_integration : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    comment : str, optional
        Specifies a comment for the stage.
    credentials : Credentials, optional

    encryption : StageEncryption, optional

    directory_table : StageDirectoryTable, optional

    created_on : datetime, optional
        Date and time when the stage was created — **Read-only:** *any user-provided value will be ignored.*
    has_credentials : bool, optional
        Indicates that the external stage has access credentials; always false for an internal stage — **Read-only:** *any user-provided value will be ignored.*
    has_encryption_key : bool, optional
        Indicates that the external stage contains encrypted files; always false for an internal stage — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the stage — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the object, either ROLE or DATABASE_ROLE. If a Snowflake Native App owns the object, the value is APPLICATION. Snowflake returns NULL if you delete the object because a deleted object does not have an owner role — **Read-only:** *any user-provided value will be ignored.*
    region : str, optional
        Region where the stage is located — **Read-only:** *any user-provided value will be ignored.*
    cloud : str, optional
        Cloud provider; always NULL for an internal stage — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    kind: Optional[StrictStr] = "PERMANENT"

    url: Optional[StrictStr] = None

    endpoint: Optional[StrictStr] = None

    storage_integration: Optional[Annotated[str, Field(strict=True)]] = None

    comment: Optional[StrictStr] = None

    credentials: Optional[Credentials] = None

    encryption: Optional[StageEncryption] = None

    directory_table: Optional[StageDirectoryTable] = None

    created_on: Optional[datetime] = None

    has_credentials: Optional[StrictBool] = None

    has_encryption_key: Optional[StrictBool] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    region: Optional[StrictStr] = None

    cloud: Optional[StrictStr] = None

    __properties = [
        "name",
        "kind",
        "url",
        "endpoint",
        "storage_integration",
        "comment",
        "credentials",
        "encryption",
        "directory_table",
        "created_on",
        "has_credentials",
        "has_encryption_key",
        "owner",
        "owner_role_type",
        "region",
        "cloud",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("kind")
    def kind_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("PERMANENT", "TEMPORARY"):
            raise ValueError("must validate the enum values ('PERMANENT','TEMPORARY')")
        return v

    @field_validator("storage_integration")
    def storage_integration_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Stage:
        """Create an instance of Stage from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "has_credentials",
                    "has_encryption_key",
                    "owner",
                    "owner_role_type",
                    "region",
                    "cloud",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of credentials
        if self.credentials:
            _dict["credentials"] = self.credentials.to_dict()

        # override the default output from pydantic by calling `to_dict()` of encryption
        if self.encryption:
            _dict["encryption"] = self.encryption.to_dict()

        # override the default output from pydantic by calling `to_dict()` of directory_table
        if self.directory_table:
            _dict["directory_table"] = self.directory_table.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Stage:
        """Create an instance of Stage from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Stage.model_validate(obj)

        _obj = Stage.model_validate(
            {
                "name": obj.get("name"),
                "kind": obj.get("kind") if obj.get("kind") is not None else "PERMANENT",
                "url": obj.get("url"),
                "endpoint": obj.get("endpoint"),
                "storage_integration": obj.get("storage_integration"),
                "comment": obj.get("comment"),
                "credentials": Credentials.from_dict(obj.get("credentials"))
                if obj.get("credentials") is not None
                else None,
                "encryption": StageEncryption.from_dict(obj.get("encryption"))
                if obj.get("encryption") is not None
                else None,
                "directory_table": StageDirectoryTable.from_dict(obj.get("directory_table"))
                if obj.get("directory_table") is not None
                else None,
                "created_on": obj.get("created_on"),
                "has_credentials": obj.get("has_credentials"),
                "has_encryption_key": obj.get("has_encryption_key"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "region": obj.get("region"),
                "cloud": obj.get("cloud"),
            }
        )

        return _obj


class StageModel:
    def __init__(
        self,
        name: str,
        # optional properties
        kind: Optional[str] = "PERMANENT",
        url: Optional[str] = None,
        endpoint: Optional[str] = None,
        storage_integration: Optional[str] = None,
        comment: Optional[str] = None,
        credentials: Optional[Credentials] = None,
        encryption: Optional[StageEncryption] = None,
        directory_table: Optional[StageDirectoryTable] = None,
        created_on: Optional[datetime] = None,
        has_credentials: Optional[bool] = None,
        has_encryption_key: Optional[bool] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        region: Optional[str] = None,
        cloud: Optional[str] = None,
    ):
        """A model object representing the Stage resource.

        Constructs an object of type Stage with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        kind : str,  default 'PERMANENT'
            Specifies whether the stage is permanent or temporary.
        url : str, optional
            URL for the external stage; blank for an internal stage.
        endpoint : str, optional
            The S3-compatible API endpoint associated with the stage; always NULL for stages that are not S3-compatible.
        storage_integration : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        comment : str, optional
            Specifies a comment for the stage.
        credentials : Credentials, optional

        encryption : StageEncryption, optional

        directory_table : StageDirectoryTable, optional

        created_on : datetime, optional
            Date and time when the stage was created.
        has_credentials : bool, optional
            Indicates that the external stage has access credentials; always false for an internal stage.
        has_encryption_key : bool, optional
            Indicates that the external stage contains encrypted files; always false for an internal stage.
        owner : str, optional
            Role that owns the stage.
        owner_role_type : str, optional
            The type of role that owns the object, either ROLE or DATABASE_ROLE. If a Snowflake Native App owns the object, the value is APPLICATION. Snowflake returns NULL if you delete the object because a deleted object does not have an owner role.
        region : str, optional
            Region where the stage is located.
        cloud : str, optional
            Cloud provider; always NULL for an internal stage.
        """
        self.name = name
        self.kind = kind
        self.url = url
        self.endpoint = endpoint
        self.storage_integration = storage_integration
        self.comment = comment
        self.credentials = credentials
        self.encryption = encryption
        self.directory_table = directory_table
        self.created_on = created_on
        self.has_credentials = has_credentials
        self.has_encryption_key = has_encryption_key
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.region = region
        self.cloud = cloud

    __properties = [
        "name",
        "kind",
        "url",
        "endpoint",
        "storage_integration",
        "comment",
        "credentials",
        "encryption",
        "directory_table",
        "created_on",
        "has_credentials",
        "has_encryption_key",
        "owner",
        "owner_role_type",
        "region",
        "cloud",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Stage(
            name=self.name,
            kind=self.kind,
            url=self.url,
            endpoint=self.endpoint,
            storage_integration=self.storage_integration,
            comment=self.comment,
            credentials=self.credentials._to_model() if self.credentials is not None else None,
            encryption=self.encryption._to_model() if self.encryption is not None else None,
            directory_table=self.directory_table._to_model() if self.directory_table is not None else None,
            created_on=self.created_on,
            has_credentials=self.has_credentials,
            has_encryption_key=self.has_encryption_key,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            region=self.region,
            cloud=self.cloud,
        )

    @classmethod
    def _from_model(cls, model) -> StageModel:
        return StageModel(
            name=model.name,
            kind=model.kind,
            url=model.url,
            endpoint=model.endpoint,
            storage_integration=model.storage_integration,
            comment=model.comment,
            credentials=CredentialsModel._from_model(model.credentials) if model.credentials else None,
            encryption=StageEncryptionModel._from_model(model.encryption) if model.encryption else None,
            directory_table=StageDirectoryTableModel._from_model(model.directory_table)
            if model.directory_table
            else None,
            created_on=model.created_on,
            has_credentials=model.has_credentials,
            has_encryption_key=model.has_encryption_key,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            region=model.region,
            cloud=model.cloud,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Stage.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Stage object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StageModel:
        """Create an instance of Stage from a dict.

        This method constructs a Stage object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Stage
            A Stage object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Stage.from_dict(obj))


Stage._model_class = StageModel



"""Manages Snowflake UserDefinedFunctions."""

from ._generated.models import (
    Argument,
    BaseLanguage,
    ColumnType,
    FunctionLanguage,
    JavaFunction,
    JavaScriptFunction,
    PythonFunction,
    ReturnDataType,
    ReturnTable,
    ReturnType,
    ScalaFunction,
    SQLFunction,
    UserDefinedFunction,
    UserDefinedFunctionArgument,
)
from ._user_defined_function import UserDefinedFunctionCollection, UserDefinedFunctionResource


__all__ = [
    "Argument",
    "BaseLanguage",
    "ColumnType",
    "FunctionLanguage",
    "JavaFunction",
    "JavaScriptFunction",
    "PythonFunction",
    "ReturnDataType",
    "ReturnTable",
    "ReturnType",
    "SQLFunction",
    "ScalaFunction",
    "UserDefinedFunction",
    "UserDefinedFunctionResource",
    "UserDefinedFunctionCollection",
    "UserDefinedFunctionArgument",
]



import pytest

from snowflake.core.pipe import Pipe
from tests.utils import random_string


@pytest.fixture
def template_pipe(temp_table, temp_stage):
    name = random_string(5, "test_create_pipe_")
    return Pipe(
        name=name,
        comment="template for pipe snowpy testing",
        # TODO(SNOW-1650785): add more attributes when they are implemented in python api
        # auto_ingest=True,
        # error_integration="dummy_error_intergration",
        # aws_sns_topic="arn:aws:sns:us-west-2:001234567890:s3_mybucket",
        # integration="dummy_integration",
        copy_statement=f"COPY into {temp_table.name} from @{temp_stage.name}",
    )


@pytest.fixture
def template_pipe_case_sensitive(temp_table, temp_stage):
    name = random_string(5, "test_create_pipe_")
    name_case_sensitive = '"' + name + '"'
    return Pipe(
        name=name_case_sensitive,
        comment="template for pipe snowpy testing",
        # TODO(SNOW-1650785): add more attributes when they are implemented in python api
        # auto_ingest=True,
        # error_integration="dummy_error_intergration",
        # aws_sns_topic="arn:aws:sns:us-west-2:001234567890:s3_mybucket",
        # integration="dummy_integration",
        copy_statement=f"COPY into {temp_table.name} from @{temp_stage.name}",
    )



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import os
import sys

import pytest

from snowflake.core import CreateMode
from snowflake.core.database import Database, DatabaseCollection
from snowflake.core.exceptions import ConflictError, NotFoundError

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_database_schema")


def test_conflict_error(databases: DatabaseCollection, root):
    root.parameters(refresh=True)
    test_database_name = random_string(3, "test_db_123")
    new_db_def = Database(name=test_database_name)
    database = databases.create(new_db_def)
    try:
        with pytest.raises(ConflictError) as exc_info:
            databases.create(new_db_def, mode=CreateMode.error_if_exists)
        assert f"Error Message: object '{test_database_name}' already exists" in str(exc_info.value)
    finally:
        database.drop()


def test_not_found_error(databases: DatabaseCollection, root):
    test_database_name = random_string(3, "db_doesnt_exist")
    with pytest.raises(NotFoundError) as exc_info:
        databases[test_database_name].fetch()
    assert f"Error Message: database '{test_database_name}' does not exist or not authorized." in str(exc_info.value)
    assert sys.tracebacklimit is None


def test_not_found_error_traceback_disabled(databases: DatabaseCollection, root):
    test_database_name = random_string(3, "db_doesnt_exist")
    os.environ["_SNOWFLAKE_PRINT_VERBOSE_STACK_TRACE"] = "false"
    root.parameters(refresh=True)
    with pytest.raises(NotFoundError) as exc_info:
        databases[test_database_name].fetch()
    assert f"Error Message: database '{test_database_name}' does not exist or not authorized." in str(exc_info.value)
    assert sys.tracebacklimit == 0
    del os.environ["_SNOWFLAKE_PRINT_VERBOSE_STACK_TRACE"]
    root.parameters(refresh=True)



from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.stream import Stream, StreamSourceTable
from tests.utils import random_string

from ...utils import ensure_snowflake_version


@pytest.fixture(scope="module")
def streams_extended(streams, src_temp_table, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.35.0")

    name_list = []
    for _ in range(5):
        name_list.append(random_string(10, "test_stream_iter_a_"))
    for _ in range(7):
        name_list.append(random_string(10, "test_stream_iter_b_"))
    for _ in range(3):
        name_list.append(random_string(10, "test_stream_iter_c_"))

    for stream_name in name_list:
        streams.create(
            Stream(
                name=stream_name, stream_source=StreamSourceTable(name=src_temp_table.name), comment="ThIs iS a ComMeNT"
            )
        )

    try:
        yield streams
    finally:
        for stream_name in name_list:
            with suppress(NotFoundError):
                streams[stream_name].drop()


def test_iter_raw(streams_extended):
    assert len(list(streams_extended.iter())) >= 15


def test_iter_like(streams_extended):
    assert len(list(streams_extended.iter(like="test_stream_iter_"))) == 0
    assert len(list(streams_extended.iter(like="test_stream_iter_a_%%"))) == 5
    assert len(list(streams_extended.iter(like="test_stream_iter_b_%%"))) == 7
    assert len(list(streams_extended.iter(like="test_stream_iter_c_%%"))) == 3


def test_iter_show_limit(streams_extended):
    assert len(list(streams_extended.iter(like="test_stream_iter_a_%%"))) == 5
    assert len(list(streams_extended.iter(like="test_stream_iter_a_%%", show_limit=2))) == 2
    assert len(list(streams_extended.iter(show_limit=2))) == 2


def test_iter_starts_with(streams_extended):
    assert len(list(streams_extended.iter(starts_with="test_stream_iter_a_".upper()))) == 5


def test_iter_from_name(streams_extended):
    assert len(list(streams_extended.iter(from_name="test_stream_iter_b_".upper(), show_limit=10000))) >= 10
    assert len(list(streams_extended.iter(from_name="test_stream_iter_b_".upper(), show_limit=5))) <= 5

    # This should return all streams
    assert len(list(streams_extended.iter(from_name="test_stream_iter_b_".upper()))) >= 15
    assert len(list(streams_extended.iter(from_name="test_stream_iter_b_", show_limit=10000))) == 0



from decimal import Decimal

import pytest

from snowflake.core.exceptions import APIError, InvalidArgumentsError
from snowflake.core.user_defined_function import (
    Argument,
    ReturnDataType,
    ScalaFunction,
    SQLFunction,
    UserDefinedFunction,
    UserDefinedFunctionArgument,
    UserDefinedFunctionResource,
)
from tests.utils import random_string


def test_execute_udf_decfloat(user_defined_functions):
    udf_name = random_string(10, "test_execute_udf_decfloat_")
    udf = UserDefinedFunction(
        name=udf_name,
        arguments=[Argument(name="arg", datatype="DECFLOAT")],
        return_type=ReturnDataType(datatype="DECFLOAT"),
        language_config=SQLFunction(),
        body="SELECT DECFLOAT '1.23e-2' + ARG",
    )
    udf_handle: UserDefinedFunctionResource = user_defined_functions.create(udf)
    try:
        result = udf_handle.execute(
            [UserDefinedFunctionArgument(name="arg", datatype="DECFLOAT", value=Decimal("1.02"))]
        )
        assert result == Decimal("1.0323")
    finally:
        udf_handle.drop(if_exists=True)


def test_execute_udf_with_two_args(user_defined_functions):
    func_body = """
                  class Echo {
                    def echoText(x : String, y: Int): String = {
                      return x
                    }
                  }
                """
    udf = UserDefinedFunction(
        name="echo_text",
        arguments=[Argument(name="x", datatype="TEXT"), Argument(name="y", datatype="INT")],
        return_type=ReturnDataType(datatype="TEXT"),
        language_config=ScalaFunction(runtime_version="2.12", handler="Echo.echoText", packages=[]),
        body=func_body,
    )
    udf_handle: UserDefinedFunctionResource = user_defined_functions.create(udf)
    try:
        result = udf_handle.execute(
            [
                UserDefinedFunctionArgument(name="y", datatype="INT", value=42),
                UserDefinedFunctionArgument(name="x", datatype="TEXT", value="TEST VALUE"),
            ]
        )
        assert result == "TEST VALUE"
    finally:
        udf_handle.drop(if_exists=True)


def test_execute_udf_without_args(user_defined_functions):
    func_body = """
              class Echo {
                def echoText(x: String): String = {
                  return x
                }
              }
            """
    udf = UserDefinedFunction(
        name="echo_text",
        arguments=[Argument(name="x", datatype="TEXT", default_value="'TEST VALUE'")],
        return_type=ReturnDataType(datatype="TEXT"),
        language_config=ScalaFunction(runtime_version="2.12", handler="Echo.echoText", packages=[]),
        body=func_body,
    )
    udf_handle: UserDefinedFunctionResource = user_defined_functions.create(udf)
    try:
        assert udf_handle.execute() == "TEST VALUE"
        assert udf_handle.execute([]) == "TEST VALUE"
    finally:
        udf_handle.drop(if_exists=True)


def test_execute_udf_invalid_argument_provided(user_defined_functions):
    func_body = """
              class Echo {
                def echoText(x : Int): Int = {
                  return x
                }
              }
            """
    udf = UserDefinedFunction(
        name="echo_text",
        arguments=[Argument(name="x", datatype="INT")],
        return_type=ReturnDataType(datatype="INT"),
        language_config=ScalaFunction(runtime_version="2.12", handler="Echo.echoText", packages=[]),
        body=func_body,
        comment="test_comment",
    )
    udf_handle: UserDefinedFunctionResource = user_defined_functions.create(udf)
    try:
        with pytest.raises(APIError):
            udf_handle.execute([UserDefinedFunctionArgument(name="x", datatype="INT", value="NOT AN INT")])
        with pytest.raises(InvalidArgumentsError):
            udf_handle.execute([UserDefinedFunctionArgument(name="y", datatype="INT", value=42)])
    finally:
        udf_handle.drop(if_exists=True)



# coding: utf-8

# flake8: noqa

"""
Snowflake Session API.

The Snowflake Session API is a REST API that you can use to query your current session properties.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.session._generated.api.session_api import SessionApi

# import models into sdk package
from snowflake.core.session._generated.models.current_secondary_roles import CurrentSecondaryRoles
from snowflake.core.session._generated.models.default_result import DefaultResult
from snowflake.core.session._generated.models.error_response import ErrorResponse
from snowflake.core.session._generated.models.named_default import NamedDefault
from snowflake.core.session._generated.models.parameter import Parameter
from snowflake.core.session._generated.models.secondary_roles import SecondaryRoles
from snowflake.core.session._generated.models.success_response import SuccessResponse



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict

from snowflake.core.cortex.search_service._generated.models.numeric_boost import NumericBoost, NumericBoostModel
from snowflake.core.cortex.search_service._generated.models.query_similarity_boost import (
    QuerySimilarityBoost,
    QuerySimilarityBoostModel,
)
from snowflake.core.cortex.search_service._generated.models.text_boost import TextBoost, TextBoostModel
from snowflake.core.cortex.search_service._generated.models.time_decay import TimeDecay, TimeDecayModel
from snowflake.core.cortex.search_service._generated.models.vector_boost import VectorBoost, VectorBoostModel


class Functions(BaseModel):
    """A model object representing the Functions resource.

    Constructs an object of type Functions with the provided properties.

    Parameters
    __________
    numeric_boosts : list[NumericBoost], optional

    time_decays : list[TimeDecay], optional

    text_boosts : list[TextBoost], optional

    vector_boosts : list[VectorBoost], optional

    query_similarity_boosts : list[QuerySimilarityBoost], optional
    """

    numeric_boosts: Optional[List[NumericBoost]] = None

    time_decays: Optional[List[TimeDecay]] = None

    text_boosts: Optional[List[TextBoost]] = None

    vector_boosts: Optional[List[VectorBoost]] = None

    query_similarity_boosts: Optional[List[QuerySimilarityBoost]] = None

    __properties = ["numeric_boosts", "time_decays", "text_boosts", "vector_boosts", "query_similarity_boosts"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Functions:
        """Create an instance of Functions from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in numeric_boosts (list)
        _items = []
        if self.numeric_boosts:
            for _item in self.numeric_boosts:
                if _item:
                    _items.append(_item.to_dict())
            _dict["numeric_boosts"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in time_decays (list)
        _items = []
        if self.time_decays:
            for _item in self.time_decays:
                if _item:
                    _items.append(_item.to_dict())
            _dict["time_decays"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in text_boosts (list)
        _items = []
        if self.text_boosts:
            for _item in self.text_boosts:
                if _item:
                    _items.append(_item.to_dict())
            _dict["text_boosts"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in vector_boosts (list)
        _items = []
        if self.vector_boosts:
            for _item in self.vector_boosts:
                if _item:
                    _items.append(_item.to_dict())
            _dict["vector_boosts"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in query_similarity_boosts (list)
        _items = []
        if self.query_similarity_boosts:
            for _item in self.query_similarity_boosts:
                if _item:
                    _items.append(_item.to_dict())
            _dict["query_similarity_boosts"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Functions:
        """Create an instance of Functions from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Functions.model_validate(obj)

        _obj = Functions.model_validate(
            {
                "numeric_boosts": [NumericBoost.from_dict(_item) for _item in obj.get("numeric_boosts")]
                if obj.get("numeric_boosts") is not None
                else None,
                "time_decays": [TimeDecay.from_dict(_item) for _item in obj.get("time_decays")]
                if obj.get("time_decays") is not None
                else None,
                "text_boosts": [TextBoost.from_dict(_item) for _item in obj.get("text_boosts")]
                if obj.get("text_boosts") is not None
                else None,
                "vector_boosts": [VectorBoost.from_dict(_item) for _item in obj.get("vector_boosts")]
                if obj.get("vector_boosts") is not None
                else None,
                "query_similarity_boosts": [
                    QuerySimilarityBoost.from_dict(_item) for _item in obj.get("query_similarity_boosts")
                ]
                if obj.get("query_similarity_boosts") is not None
                else None,
            }
        )

        return _obj


class FunctionsModel:
    def __init__(
        self,  # optional properties
        numeric_boosts: Optional[list[NumericBoost]] = None,
        time_decays: Optional[list[TimeDecay]] = None,
        text_boosts: Optional[list[TextBoost]] = None,
        vector_boosts: Optional[list[VectorBoost]] = None,
        query_similarity_boosts: Optional[list[QuerySimilarityBoost]] = None,
    ):
        """A model object representing the Functions resource.

        Constructs an object of type Functions with the provided properties.

        Parameters
        __________
        numeric_boosts : list[NumericBoost], optional

        time_decays : list[TimeDecay], optional

        text_boosts : list[TextBoost], optional

        vector_boosts : list[VectorBoost], optional

        query_similarity_boosts : list[QuerySimilarityBoost], optional
        """
        self.numeric_boosts = numeric_boosts
        self.time_decays = time_decays
        self.text_boosts = text_boosts
        self.vector_boosts = vector_boosts
        self.query_similarity_boosts = query_similarity_boosts

    __properties = ["numeric_boosts", "time_decays", "text_boosts", "vector_boosts", "query_similarity_boosts"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Functions(
            numeric_boosts=[x._to_model() for x in self.numeric_boosts] if self.numeric_boosts is not None else None,
            time_decays=[x._to_model() for x in self.time_decays] if self.time_decays is not None else None,
            text_boosts=[x._to_model() for x in self.text_boosts] if self.text_boosts is not None else None,
            vector_boosts=[x._to_model() for x in self.vector_boosts] if self.vector_boosts is not None else None,
            query_similarity_boosts=[x._to_model() for x in self.query_similarity_boosts]
            if self.query_similarity_boosts is not None
            else None,
        )

    @classmethod
    def _from_model(cls, model) -> FunctionsModel:
        return FunctionsModel(
            numeric_boosts=[NumericBoostModel._from_model(x) for x in model.numeric_boosts]
            if model.numeric_boosts
            else None,
            time_decays=[TimeDecayModel._from_model(x) for x in model.time_decays] if model.time_decays else None,
            text_boosts=[TextBoostModel._from_model(x) for x in model.text_boosts] if model.text_boosts else None,
            vector_boosts=[VectorBoostModel._from_model(x) for x in model.vector_boosts]
            if model.vector_boosts
            else None,
            query_similarity_boosts=[QuerySimilarityBoostModel._from_model(x) for x in model.query_similarity_boosts]
            if model.query_similarity_boosts
            else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Functions.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Functions object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> FunctionsModel:
        """Create an instance of Functions from a dict.

        This method constructs a Functions object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Functions
            A Functions object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Functions.from_dict(obj))


Functions._model_class = FunctionsModel



from typing import Optional

import pytest

from snowflake.core import CreateMode
from snowflake.core.dynamic_table import (
    DownstreamLag,
    DynamicTable,
    DynamicTableCollection,
    DynamicTableColumn,
)
from snowflake.core.table import TableResource
from snowflake.core.tag import TagResource, TagValue
from snowflake.core.warehouse import WarehouseResource

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


class TestDynamicTableTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, dynamic_tables: DynamicTableCollection, warehouse: WarehouseResource, temp_table: TableResource):
        self.dynamic_tables = dynamic_tables
        dynamic_table_name = random_string(6, "dynamic_table_for_tag_tests_")
        dynamic_table = DynamicTable(
            name=dynamic_table_name,
            warehouse=warehouse.name,
            target_lag=DownstreamLag(),
            columns=[DynamicTableColumn(name="c1"), DynamicTableColumn(name="c2")],
            query=f"SELECT * FROM {temp_table.name}",
        )
        self.dynamic_table_res = dynamic_tables.create(dynamic_table, mode=CreateMode.if_not_exists)
        try:
            yield
        finally:
            self.dynamic_table_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "TABLE"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.dynamic_tables[resource_name or self.dynamic_table_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.dynamic_tables[resource_name or self.dynamic_table_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.dynamic_tables[resource_name or self.dynamic_table_res.name].get_tags(with_lineage)



"""
Snowflake Notebook API.

The Snowflake Notebook API is a REST API that you can use to access, update, and perform certain actions on Notebook resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class VersionDetails(BaseModel):
    """A model object representing the VersionDetails resource.

    Constructs an object of type VersionDetails with the provided properties.

    Parameters
    __________
    name : str, optional
        The last version name — **Read-only:** *any user-provided value will be ignored.*
    alias : str, optional
        The default/last version alias of a file based entity — **Read-only:** *any user-provided value will be ignored.*
    location_url : str, optional
        The default/last version location — **Read-only:** *any user-provided value will be ignored.*
    source_location_uri : str, optional
        The default/last version source location — **Read-only:** *any user-provided value will be ignored.*
    git_commit_hash : str, optional
        The default/last version git commit# — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Optional[StrictStr] = None

    alias: Optional[StrictStr] = None

    location_url: Optional[StrictStr] = None

    source_location_uri: Optional[StrictStr] = None

    git_commit_hash: Optional[StrictStr] = None

    __properties = ["name", "alias", "location_url", "source_location_uri", "git_commit_hash"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VersionDetails:
        """Create an instance of VersionDetails from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "name",
                    "alias",
                    "location_url",
                    "source_location_uri",
                    "git_commit_hash",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> VersionDetails:
        """Create an instance of VersionDetails from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return VersionDetails.model_validate(obj)

        _obj = VersionDetails.model_validate(
            {
                "name": obj.get("name"),
                "alias": obj.get("alias"),
                "location_url": obj.get("location_url"),
                "source_location_uri": obj.get("source_location_uri"),
                "git_commit_hash": obj.get("git_commit_hash"),
            }
        )

        return _obj


class VersionDetailsModel:
    def __init__(
        self,  # optional properties
        name: Optional[str] = None,
        alias: Optional[str] = None,
        location_url: Optional[str] = None,
        source_location_uri: Optional[str] = None,
        git_commit_hash: Optional[str] = None,
    ):
        """A model object representing the VersionDetails resource.

        Constructs an object of type VersionDetails with the provided properties.

        Parameters
        __________
        name : str, optional
            The last version name
        alias : str, optional
            The default/last version alias of a file based entity.
        location_url : str, optional
            The default/last version location.
        source_location_uri : str, optional
            The default/last version source location.
        git_commit_hash : str, optional
            The default/last version git commit#
        """
        self.name = name
        self.alias = alias
        self.location_url = location_url
        self.source_location_uri = source_location_uri
        self.git_commit_hash = git_commit_hash

    __properties = ["name", "alias", "location_url", "source_location_uri", "git_commit_hash"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return VersionDetails(
            name=self.name,
            alias=self.alias,
            location_url=self.location_url,
            source_location_uri=self.source_location_uri,
            git_commit_hash=self.git_commit_hash,
        )

    @classmethod
    def _from_model(cls, model) -> VersionDetailsModel:
        return VersionDetailsModel(
            name=model.name,
            alias=model.alias,
            location_url=model.location_url,
            source_location_uri=model.source_location_uri,
            git_commit_hash=model.git_commit_hash,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a VersionDetails.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the VersionDetails object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> VersionDetailsModel:
        """Create an instance of VersionDetails from a dict.

        This method constructs a VersionDetails object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        VersionDetails
            A VersionDetails object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(VersionDetails.from_dict(obj))


VersionDetails._model_class = VersionDetailsModel



from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.streamlit import Streamlit
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("9.38.0")
def test_create_and_fetch(streamlits, streamlit_stage_with_file, warehouse, streamlit_main_file):
    name = random_string(8, "test_streamlit_")
    handle = streamlits[name]

    st = Streamlit(
        name=name,
        query_warehouse=warehouse.name,
        source_location=f"@{streamlit_stage_with_file.name}",
        main_file=streamlit_main_file,
        comment="Test streamlit for integration testing",
        title="Test Streamlit App",
    )

    res = streamlits.create(st)
    try:
        fetched = handle.fetch()
        assert fetched.name.upper() == name.upper()
        assert fetched.last_version_details is not None
        assert fetched.last_version_details.source_location_uri.upper() == f"@{streamlit_stage_with_file.name}".upper()
        assert fetched.main_file.upper() == streamlit_main_file.upper()
        assert fetched.query_warehouse.upper() == warehouse.name.upper()
        assert fetched.comment == "Test streamlit for integration testing"
        assert fetched.title == "Test Streamlit App"
    finally:
        with suppress(NotFoundError):
            res.drop()



"""Manages Snowflake sequence."""

from ._generated.models import (
    Sequence,
)
from ._sequence import SequenceCollection, SequenceResource


__all__ = [
    "SequenceResource",
    "SequenceCollection",
    "Sequence",
]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.

from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_rename(warehouses, session):
    warehouse_name = random_string(5, "test_rename_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name)

    warehouse_new_name = random_string(5, "test_new_name_warehouse_")
    assert warehouse_name != warehouse_new_name

    warehouse_ref = None
    try:
        # Rename the warehouse to a new name.
        warehouse_ref = warehouses.create(test_warehouse)
        warehouse_ref.rename(warehouse_new_name)
        result1 = warehouses.iter(like=warehouse_name)
        result2 = warehouses.iter(like=warehouse_new_name)
        assert len(list(result1)) == 0 and len(list(result2)) == 1

        # Change the warehouse name back.
        warehouses[warehouse_new_name].rename(warehouse_name)
        result1 = warehouses.iter(like=warehouse_name)
        result2 = warehouses.iter(like=warehouse_new_name)
        assert len(list(result1)) == 1 and len(list(result2)) == 0

        warehouse_ref.drop()

        # Should not error.
        warehouse_ref.rename(warehouse_new_name, if_exists=True)
    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()



import io
import json
import logging
import typing


if typing.TYPE_CHECKING:
    from urllib3.response import HTTPHeaderDict, HTTPResponse  # type: ignore[attr-defined]

logger = logging.getLogger(__name__)
_FIELD_SEPARATOR = ":"


class RESTResponse(io.IOBase):
    def __init__(self, resp: "HTTPResponse") -> None:
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data
        logger.debug("created a RESTResponse with status %d and length of %d", self.status, len(self.data))

    def getheaders(self) -> "HTTPHeaderDict":
        """Return a dictionary of the response headers."""
        return self.urllib3_response.headers

    def getheader(self, name: str, default: typing.Any = None) -> typing.Any:
        """Return a given response header."""
        return self.urllib3_response.headers.get(name, default)


class SSEClient:
    """Implementation of a SSE client."""

    def __init__(self, event_source: typing.Any, char_enc: str = "utf-8") -> None:
        """Initialize the SSE client over an existing, ready to consume event source.

        The event source is expected to be a binary stream and have a close()
        method. That would usually be something that implements
        io.BinaryIOBase, like an httplib or urllib3 HTTPResponse object.
        """
        logger.debug("Initialized SSE client from event source %s", event_source)
        self._event_source = event_source
        self._char_enc = char_enc

    def _read(self) -> typing.Generator[bytes, None, None]:
        r"""Read the incoming event source stream and yield event chunks.

        The event source might be a server side event stream, which looks like:

        >>> event: message.content.delta\n
        >>> data: {"type":"text","text_delta":"text blob 1","index":0}\n\n

        or it might be an array of JSON:

        >>> [
        ...     {
        ...         "event": "message.content.delta",
        ...         "data": {"type": "text", "text_delta": "text blob 1", "index": 0},
        ...     },
        ...     ...,
        ... ]
        """
        data = b""
        for chunk in self._event_source:
            for line in chunk.splitlines(True):
                data += line
                if data.endswith((b"\r\r", b"\n\n", b"\r\n\r\n")):
                    yield data
                    data = b""
        if data:
            yield data

    def events(self) -> typing.Generator["Event", None, None]:
        content_type = self._event_source.headers.get("Content-Type")
        if content_type == "text/event-stream":
            logger.debug("Handling as SSE stream.")
            return self._handle_sse()
        elif content_type and content_type.startswith("application/json"):
            # content_type can also be of type 'application/json;charset=utf-8' so we need to use startswith() here.
            logger.debug("Handling as JSON response.")
            return self._handle_json()
        else:
            raise ValueError(f"Unknown Content-Type: {content_type}. Data: {self._event_source.data!r}")

    def _handle_sse(self) -> typing.Generator["Event", None, None]:
        for chunk in self._read():
            event = Event()
            # Split before decoding so splitlines() only uses \r and \n
            for line_bytes in chunk.splitlines():
                # Decode the line.
                line = line_bytes.decode(self._char_enc)

                # Lines starting with a separator are comments and are to be
                # ignored.
                if not line.strip() or line.startswith(_FIELD_SEPARATOR):
                    continue

                data = line.split(_FIELD_SEPARATOR, 1)
                field = data[0]

                # Ignore unknown fields.
                if field not in event.__dict__:
                    logger.debug("Saw invalid field %s while parsing Server Side Event", field)
                    continue

                if len(data) > 1:
                    # From the spec:
                    # "If value starts with a single U+0020 SPACE character,
                    # remove it from value."
                    if data[1].startswith(" "):
                        value = data[1][1:]
                    else:
                        value = data[1]
                else:
                    # If no value is present after the separator,
                    # assume an empty value.
                    value = ""

                # The data field may come over multiple lines and their values
                # are concatenated with each other.
                if field == "data":
                    event.__dict__[field] += value + "\n"
                else:
                    event.__dict__[field] = value

            # Events with no data are not dispatched.
            if not event.data:
                continue

            # If the data field ends with a newline, remove it.
            if event.data.endswith("\n"):
                event.data = event.data[0:-1]

            # Empty event names default to 'message'
            event.event = event.event or "message"

            # Dispatch the event
            logger.debug("Dispatching %s...", event)
            yield event

    def _handle_json(self) -> typing.Generator["Event", None, None]:
        data_list = json.loads(self._event_source.data.decode(self._char_enc))
        if isinstance(data_list, dict):
            # For complete API which goes through XP path, data_list is a dictionary which looks like
            # {'choices': [{'message': {'content': 'Hello! \n'}}]}
            # We do not have any id, event, comment or retry fields.
            yield Event(data=json.dumps(data_list))
        elif isinstance(data_list, list):
            for data in data_list:
                yield Event(
                    id=data.get("id"),
                    event=data.get("event"),
                    data=data.get("data"),
                    comment=data.get("comment"),
                    retry=data.get("retry"),
                )

    def close(self) -> None:
        """Manually close the event source stream."""
        self._event_source.close()


class Event:
    """Representation of an event from the event stream."""

    def __init__(
        self,
        id: typing.Optional[str] = None,
        event: typing.Optional[str] = "message",
        data: typing.Optional[str] = "",
        comment: typing.Optional[str] = None,
        retry: typing.Optional[int] = None,
    ) -> None:
        self.id = id
        self.event = event
        self.data = data
        self.comment = comment
        self.retry = retry

    def __str__(self) -> str:
        s = f"{self.event} event"
        if self.id:
            s += f" #{self.id}"
        if self.data:
            s += ", {} byte{}".format(len(self.data), "s" if len(self.data) else "")
        else:
            s += ", no data"
        if self.comment:
            s += f", comment: {self.comment}"
        if self.retry:
            s += f", retry in {self.retry}ms"
        return s



from contextlib import suppress

import pytest

from snowflake.core._common import CreateMode
from snowflake.core.alert import Alert, MinutesSchedule
from snowflake.core.exceptions import ConflictError, NotFoundError
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.36.0")
def test_create_and_fetch(alerts):
    alert_name = random_string(10, "test_alert_")

    try:
        alert = alerts.create(
            Alert(
                name=alert_name,
                condition="SELECT 1",
                action="SELECT 2",
                schedule=MinutesSchedule(minutes=1),
                comment="asdf",
            )
        )

        alert_handle = alert.fetch()

        assert alert_handle.name.upper() == alert_name.upper()
        assert alert_handle.condition == "SELECT 1"
        assert alert_handle.action == "SELECT 2"
        assert alert_handle.schedule.minutes == 1
        assert alert_handle.comment == "asdf"
    finally:
        with suppress(NotFoundError):
            alerts[alert_name].drop()


@pytest.mark.min_sf_ver("8.36.0")
def test_create_and_update(alerts):
    alert_name = random_string(10, "test_alert_")

    try:
        alerts.create(
            Alert(
                name=alert_name,
                condition="SELECT 1",
                action="SELECT 2",
                schedule=MinutesSchedule(minutes=1),
                comment="asdf",
            )
        )

        with pytest.raises(ConflictError):
            alerts.create(
                Alert(
                    name=alert_name,
                    condition="SELECT 1",
                    action="SELECT 2",
                    schedule=MinutesSchedule(minutes=1),
                    comment="asdf",
                )
            )

        alerts.create(
            Alert(
                name=alert_name,
                condition="SELECT 5",
                action="SELECT 6",
                schedule=MinutesSchedule(minutes=7),
                comment="fdsa",
            ),
            mode=CreateMode.or_replace,
        )

        alert_handle = alerts[alert_name].fetch()

        assert alert_handle.name.upper() == alert_name.upper()
        assert alert_handle.condition == "SELECT 5"
        assert alert_handle.action == "SELECT 6"
        assert alert_handle.schedule.minutes == 7
        assert alert_handle.comment == "fdsa"
    finally:
        with suppress(NotFoundError):
            alerts[alert_name].drop()



# coding: utf-8

# flake8: noqa

"""
Snowflake Dynamic Table API.

The Snowflake Dynamic Table API is a REST API that you can use to access, update, and perform certain actions on Dynamic Tables resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.dynamic_table._generated.api.dynamic_table_api import DynamicTableApi

# import models into sdk package
from snowflake.core.dynamic_table._generated.models.downstream_lag import DownstreamLag
from snowflake.core.dynamic_table._generated.models.dynamic_table import DynamicTable
from snowflake.core.dynamic_table._generated.models.dynamic_table_clone import DynamicTableClone
from snowflake.core.dynamic_table._generated.models.dynamic_table_column import DynamicTableColumn
from snowflake.core.dynamic_table._generated.models.error_response import ErrorResponse
from snowflake.core.dynamic_table._generated.models.point_of_time import PointOfTime
from snowflake.core.dynamic_table._generated.models.point_of_time_offset import PointOfTimeOffset
from snowflake.core.dynamic_table._generated.models.point_of_time_statement import PointOfTimeStatement
from snowflake.core.dynamic_table._generated.models.point_of_time_timestamp import PointOfTimeTimestamp
from snowflake.core.dynamic_table._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.dynamic_table._generated.models.success_response import SuccessResponse
from snowflake.core.dynamic_table._generated.models.tag_assignment import TagAssignment
from snowflake.core.dynamic_table._generated.models.tag_reference import TagReference
from snowflake.core.dynamic_table._generated.models.target_lag import TargetLag
from snowflake.core.dynamic_table._generated.models.user_defined_lag import UserDefinedLag



import copy

import pytest

from snowflake.core import CreateMode
from snowflake.core.exceptions import ConflictError
from tests.integ.utils import random_string

from .conftest import (
    test_password_policy_minimal_template,
    test_password_policy_template,
)


def test_create_and_fetch(password_policies):
    name = random_string(10, "test_password_policy_create_and_fetch_")
    password_policy_handle = password_policies[name]

    try:
        password_policy = copy.deepcopy(test_password_policy_template)
        password_policy.name = name
        password_policies.create(password_policy)

        fetched_policy = password_policy_handle.fetch()

        assert fetched_policy.name.upper() == name.upper()
        assert fetched_policy.password_min_length == 8
        assert fetched_policy.password_max_length == 256
        assert fetched_policy.password_min_upper_case_chars == 1
        assert fetched_policy.password_min_lower_case_chars == 1
        assert fetched_policy.password_min_numeric_chars == 1
        assert fetched_policy.password_min_special_chars == 1
        assert fetched_policy.password_min_age_days == 1
        assert fetched_policy.password_max_age_days == 90
        assert fetched_policy.password_max_retries == 5
        assert fetched_policy.password_lockout_time_mins == 15
        assert fetched_policy.password_history == 24
        assert fetched_policy.comment == "Test password policy"
    finally:
        password_policy_handle.drop(if_exists=True)


def test_create_and_fetch_minimal(password_policies):
    name = random_string(10, "test_password_policy_create_and_fetch_")
    password_policy_handle = password_policies[name]

    try:
        password_policy = copy.deepcopy(test_password_policy_minimal_template)
        password_policy.name = name
        password_policies.create(password_policy)

        fetched_policy = password_policy_handle.fetch()

        assert fetched_policy.name.upper() == name.upper()
        assert fetched_policy.password_min_length == 14  # Default value
    finally:
        password_policy_handle.drop(if_exists=True)


def test_create_and_fetch_create_modes(password_policies):
    name = random_string(10, "test_password_policy_create_and_fetch_")
    password_policy_handle = password_policies[name]

    try:
        password_policy = copy.deepcopy(test_password_policy_template)
        password_policy.name = name
        password_policy.comment = "First version"
        password_policies.create(password_policy, mode=CreateMode.error_if_exists)
        assert password_policy_handle.fetch().comment == "First version"

        with pytest.raises(ConflictError):
            password_policies.create(password_policy, mode=CreateMode.error_if_exists)

        password_policy.comment = "Second version"
        password_policies.create(password_policy, mode=CreateMode.or_replace)
        assert password_policy_handle.fetch().comment == "Second version"

        password_policy.comment = "Should not change"
        password_policies.create(password_policy, mode=CreateMode.if_not_exists)
        assert password_policy_handle.fetch().comment == "Second version"
    finally:
        password_policy_handle.drop(if_exists=True)



import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import ConflictError, NotFoundError
from snowflake.core.notification_integration import (
    NotificationEmail,
    NotificationIntegration,
    NotificationQueueAwsSnsOutbound,
    NotificationQueueAzureEventGridInbound,
    NotificationQueueAzureEventGridOutbound,
    NotificationQueueGcpPubsubInbound,
    NotificationQueueGcpPubsubOutbound,
    NotificationWebhook,
    WebhookSecret,
)

from ..utils import random_string


# Note: all secret looking things in this file are made up

pytestmark = [
    # All tests in this file need this Snowflake version
    pytest.mark.min_sf_ver("8.36.0"),
    pytest.mark.internal_only,
]


@pytest.mark.use_accountadmin
def test_ni_creation_and_drop_notificationemail(notification_integrations, set_internal_params):
    allowed_recipients = ["test1@snowflake.com", "test2@snowflake.com"]
    default_recipients = ["test1@snowflake.com"]
    default_subject = "test default subject"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_"),
        notification_hook=NotificationEmail(
            allowed_recipients=allowed_recipients,
            default_recipients=default_recipients,
            default_subject=default_subject,
        ),
        enabled=True,
    )
    with set_internal_params({"ENABLE_LIMIT_RECIPIENTS_TO_SENDING_ACCOUNT": False}):
        pre_create_count = len(list(notification_integrations.iter()))
        ni = notification_integrations.create(new_integration)
        with pytest.raises(ConflictError):
            ni = notification_integrations.create(new_integration, mode=CreateMode.error_if_exists)
        fetched_ni = ni.fetch()
        assert fetched_ni.name == new_integration.name.upper()
        assert fetched_ni.enabled == new_integration.enabled
        assert fetched_ni.comment is None
        assert isinstance(fetched_ni.notification_hook, NotificationEmail)
        assert fetched_ni.notification_hook.allowed_recipients == allowed_recipients
        assert fetched_ni.notification_hook.default_recipients == default_recipients
        assert fetched_ni.notification_hook.default_subject == default_subject
        created_count = len(list(notification_integrations.iter()))
        ni.drop(if_exists=False)
        with pytest.raises(NotFoundError):
            ni.fetch()
        ni.drop(if_exists=True)
        with pytest.raises(NotFoundError):
            ni.drop(if_exists=False)
        after_drop_count = len(list(notification_integrations.iter()))
        assert pre_create_count + 1 == created_count == after_drop_count + 1


@pytest.mark.use_accountadmin
def test_ni_creation_and_drop_webhook(notification_integrations, cursor):
    webhook_url = "https://events.pagerduty.com/v2/enqueue"
    webhook_template = '{"key": "SNOWFLAKE_WEBHOOK_SECRET", "msg": "SNOWFLAKE_WEBHOOK_MESSAGE"}'
    webhook_headers = {"content-type": "application/json", "user-content": "chrome"}
    database = cursor.execute("select current_database();").fetchone()[0]
    schema = cursor.execute("select current_schema();").fetchone()[0]
    cursor.execute("CREATE OR REPLACE SECRET mySecret TYPE=GENERIC_STRING SECRET_STRING='aaa'")
    try:
        new_integration = NotificationIntegration(
            name=random_string(3, "test_ni_creation_and_drop_webhook_"),
            enabled=False,
            notification_hook=NotificationWebhook(
                webhook_url=webhook_url,
                webhook_secret=WebhookSecret(name="mySecret".upper(), database_name=database, schema_name=schema),
                webhook_body_template=webhook_template,
                webhook_headers=webhook_headers,
            ),
        )
        pre_create_count = len(list(notification_integrations.iter()))
        ni = notification_integrations.create(new_integration)
        ni = notification_integrations.create(new_integration, mode=CreateMode.or_replace)
        fetched_ni = ni.fetch()
        assert fetched_ni.name == new_integration.name.upper()
        assert fetched_ni.enabled == new_integration.enabled
        assert fetched_ni.comment is None
        assert isinstance(fetched_ni.notification_hook, NotificationWebhook)
        assert fetched_ni.notification_hook.webhook_url == webhook_url
        assert fetched_ni.notification_hook.webhook_body_template == webhook_template
        assert fetched_ni.notification_hook.webhook_headers == webhook_headers
        assert fetched_ni.notification_hook.webhook_secret.name == "MYSECRET"
        assert fetched_ni.notification_hook.webhook_secret.database_name == database
        assert fetched_ni.notification_hook.webhook_secret.schema_name == schema
        created_count = len(list(notification_integrations.iter()))
        ni.drop(if_exists=False)
        with pytest.raises(NotFoundError):
            ni.fetch()
        ni.drop(if_exists=True)
        with pytest.raises(NotFoundError):
            ni.drop(if_exists=False)
        after_drop_count = len(list(notification_integrations.iter()))
        assert pre_create_count + 1 == created_count == after_drop_count + 1
        # We should be able to create the integration again now
        ni = notification_integrations.create(new_integration)
        ni.drop()
    finally:
        cursor.execute("DROP SECRET mySecret;")


def test_ni_creation_and_drop_awsoutbound(notification_integrations, setup_credentials_fixture):
    del setup_credentials_fixture
    aws_sns_topic_arn = "arn:aws:sns:us-west-1:234567812345:sns-test-topic"
    aws_sns_role_arn = "arn:aws:iam::234567812345:role/sns-test-topic"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_awsoutbound_"),
        enabled=False,
        notification_hook=NotificationQueueAwsSnsOutbound(
            aws_sns_topic_arn=aws_sns_topic_arn, aws_sns_role_arn=aws_sns_role_arn
        ),
    )
    pre_create_count = len(list(notification_integrations.iter()))
    ni = notification_integrations.create(new_integration)
    fetched_ni = ni.fetch()
    assert fetched_ni.name == new_integration.name.upper()
    assert fetched_ni.enabled == new_integration.enabled
    assert fetched_ni.comment is None
    assert isinstance(fetched_ni.notification_hook, NotificationQueueAwsSnsOutbound)
    assert fetched_ni.notification_hook.sf_aws_iam_user_arn is not None
    assert fetched_ni.notification_hook.sf_aws_external_id is not None
    assert fetched_ni.notification_hook.aws_sns_topic_arn == aws_sns_topic_arn
    assert fetched_ni.notification_hook.aws_sns_role_arn == aws_sns_role_arn
    created_count = len(list(notification_integrations.iter()))
    ni.drop(if_exists=False)
    with pytest.raises(NotFoundError):
        ni.fetch()
    ni.drop(if_exists=True)
    with pytest.raises(NotFoundError):
        ni.drop(if_exists=False)
    after_drop_count = len(list(notification_integrations.iter()))
    assert pre_create_count + 1 == created_count == after_drop_count + 1


def test_ni_creation_and_drop_azureoutbound(notification_integrations, setup_credentials_fixture):
    del setup_credentials_fixture
    azure_event_grid_topic_endpoint = "https://test-snowapi-eventgrid-toopic.westus-1.eventgrid.azure.net/api/events"
    azure_tenant_id = "fake.azsnowdevoutlook.onmicrosoft.com"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_azureoutbound_"),
        enabled=False,
        notification_hook=NotificationQueueAzureEventGridOutbound(
            azure_event_grid_topic_endpoint=azure_event_grid_topic_endpoint, azure_tenant_id=azure_tenant_id
        ),
    )
    pre_create_count = len(list(notification_integrations.iter()))
    ni = notification_integrations.create(new_integration)
    fetched_ni = ni.fetch()
    assert fetched_ni.name == new_integration.name.upper()
    assert fetched_ni.enabled == new_integration.enabled
    assert fetched_ni.comment is None
    assert isinstance(fetched_ni.notification_hook, NotificationQueueAzureEventGridOutbound)
    assert fetched_ni.notification_hook.azure_consent_url is not None
    assert fetched_ni.notification_hook.azure_multi_tenant_app_name is not None
    assert fetched_ni.notification_hook.azure_event_grid_topic_endpoint == azure_event_grid_topic_endpoint
    assert fetched_ni.notification_hook.azure_tenant_id == azure_tenant_id
    created_count = len(list(notification_integrations.iter()))
    ni.drop(if_exists=False)
    with pytest.raises(NotFoundError):
        ni.fetch()
    ni.drop(if_exists=True)
    with pytest.raises(NotFoundError):
        ni.drop(if_exists=False)
    after_drop_count = len(list(notification_integrations.iter()))
    assert pre_create_count + 1 == created_count == after_drop_count + 1


def test_ni_creation_and_drop_azureinbound(notification_integrations, setup_credentials_fixture):
    del setup_credentials_fixture
    azure_storage_queue_primary_uri = "https://fake.queue.core.windows.net/snowapi_queue"
    azure_tenant_id = "fake.onmicrosoft.com"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_azureoutbound_"),
        enabled=False,
        notification_hook=NotificationQueueAzureEventGridInbound(
            azure_storage_queue_primary_uri=azure_storage_queue_primary_uri, azure_tenant_id=azure_tenant_id
        ),
    )
    pre_create_count = len(list(notification_integrations.iter()))
    ni = notification_integrations.create(new_integration)
    fetched_ni = ni.fetch()
    assert fetched_ni.name == new_integration.name.upper()
    assert fetched_ni.enabled == new_integration.enabled
    assert fetched_ni.comment is None
    assert isinstance(fetched_ni.notification_hook, NotificationQueueAzureEventGridInbound)
    assert fetched_ni.notification_hook.azure_consent_url is not None
    assert fetched_ni.notification_hook.azure_multi_tenant_app_name is not None
    assert fetched_ni.notification_hook.azure_storage_queue_primary_uri == azure_storage_queue_primary_uri
    assert fetched_ni.notification_hook.azure_tenant_id == azure_tenant_id
    created_count = len(list(notification_integrations.iter()))
    ni.drop(if_exists=False)
    with pytest.raises(NotFoundError):
        ni.fetch()
    ni.drop(if_exists=True)
    with pytest.raises(NotFoundError):
        ni.drop(if_exists=False)
    after_drop_count = len(list(notification_integrations.iter()))
    assert pre_create_count + 1 == created_count == after_drop_count + 1


def test_ni_creation_and_drop_gcpoutbound(notification_integrations, setup_credentials_fixture):
    del setup_credentials_fixture
    gcp_pubsub_topic_name = "projects/fake-project-name/topics/pythonapi-test"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_gcpoutbound_"),
        enabled=False,
        notification_hook=NotificationQueueGcpPubsubOutbound(gcp_pubsub_topic_name=gcp_pubsub_topic_name),
    )
    pre_create_count = len(list(notification_integrations.iter()))
    ni = notification_integrations.create(new_integration)
    fetched_ni = ni.fetch()
    assert fetched_ni.name == new_integration.name.upper()
    assert fetched_ni.enabled == new_integration.enabled
    assert fetched_ni.comment is None
    assert isinstance(fetched_ni.notification_hook, NotificationQueueGcpPubsubOutbound)
    assert fetched_ni.notification_hook.gcp_pubsub_service_account is not None
    assert fetched_ni.notification_hook.gcp_pubsub_topic_name == gcp_pubsub_topic_name
    created_count = len(list(notification_integrations.iter()))
    ni.drop(if_exists=False)
    with pytest.raises(NotFoundError):
        ni.fetch()
    ni.drop(if_exists=True)
    with pytest.raises(NotFoundError):
        ni.drop(if_exists=False)
    after_drop_count = len(list(notification_integrations.iter()))
    assert pre_create_count + 1 == created_count == after_drop_count + 1


def test_ni_creation_and_drop_gcpinbound(notification_integrations, setup_credentials_fixture):
    del setup_credentials_fixture
    gcp_pubsub_subscription_name = "projects/snowapi-snowfort-project/subscriptions/sub2"
    new_integration = NotificationIntegration(
        name=random_string(3, "test_ni_creation_and_drop_gcpoutbound_"),
        enabled=False,
        notification_hook=NotificationQueueGcpPubsubInbound(gcp_pubsub_subscription_name=gcp_pubsub_subscription_name),
    )
    pre_create_count = len(list(notification_integrations.iter()))
    ni = notification_integrations.create(new_integration)
    fetched_ni = ni.fetch()
    assert fetched_ni.name == new_integration.name.upper()
    assert fetched_ni.enabled == new_integration.enabled
    assert fetched_ni.comment is None
    assert isinstance(fetched_ni.notification_hook, NotificationQueueGcpPubsubInbound)
    assert fetched_ni.notification_hook.gcp_pubsub_service_account is not None
    assert fetched_ni.notification_hook.gcp_pubsub_subscription_name == gcp_pubsub_subscription_name
    created_count = len(list(notification_integrations.iter()))
    ni.drop(if_exists=False)
    with pytest.raises(NotFoundError):
        ni.fetch()
    ni.drop(if_exists=True)
    with pytest.raises(NotFoundError):
        ni.drop(if_exists=False)
    after_drop_count = len(list(notification_integrations.iter()))
    assert pre_create_count + 1 == created_count == after_drop_count + 1



import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.stage import Stage
from tests.utils import random_string


def test_drop(stages):
    comment = "my comment"
    new_stage = Stage(name=random_string(5, "test_stage_"), comment=comment)
    try:
        s = stages.create(new_stage)
        assert s.fetch().comment == comment
    finally:
        s.drop()

    with pytest.raises(NotFoundError):
        s.fetch()
    try:
        s = stages.create(new_stage)
        assert s.fetch().comment == comment
    finally:
        s.drop()

    # check if exists
    with pytest.raises(NotFoundError):
        stages["RANDOM"].drop()

    stages["RANDOM"].drop(if_exists=True)



import datetime

from snowflake.core._internal.utils import normalize_and_unquote_name, normalize_datatype
from snowflake.core.database import DatabaseResource
from snowflake.core.dynamic_table import DownstreamLag, DynamicTable
from snowflake.core.schema import SchemaResource
from snowflake.core.table import TableResource


def assert_dynamic_table(
    dynamic_table: DynamicTable,
    name: str,
    table: TableResource,
    database: DatabaseResource,
    schema: SchemaResource,
    db_parameters: dict[str, str],
    deep: bool = False,
) -> None:
    assert dynamic_table.name == normalize_and_unquote_name(name)
    assert dynamic_table.target_lag == DownstreamLag()
    assert dynamic_table.warehouse == normalize_and_unquote_name(db_parameters["warehouse"])

    if deep:
        assert len(dynamic_table.columns) == 2
        assert dynamic_table.columns[0].name == "A"
        assert dynamic_table.columns[0].datatype == normalize_datatype("int")
        assert dynamic_table.columns[0].comment is None
        assert dynamic_table.columns[1].name == "B"
        assert dynamic_table.columns[1].datatype == normalize_datatype("varchar")
        assert dynamic_table.columns[1].comment == "comment"
    else:
        assert dynamic_table.columns is None

    assert dynamic_table.query == f"SELECT * FROM {table.name}"
    assert dynamic_table.initialize == "ON_SCHEDULE"
    assert dynamic_table.scheduling_state == "RUNNING"
    assert dynamic_table.data_retention_time_in_days == 1
    assert dynamic_table.max_data_extension_time_in_days == 2
    assert dynamic_table.automatic_clustering is True
    assert dynamic_table.cluster_by == ["B"]
    assert dynamic_table.comment == "test table", dynamic_table.comment

    assert dynamic_table.owner_role_type is not None
    assert isinstance(dynamic_table.created_on, datetime.datetime)
    assert dynamic_table.database_name == normalize_and_unquote_name(database.name)
    assert dynamic_table.schema_name == normalize_and_unquote_name(schema.name)



import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("8.34.0")
@pytest.mark.use_accountadmin
def test_drop(network_policies, template_network_policy):
    network_policy_name = random_string(10, "test_network_policy_")
    network_policy_handle = network_policies[network_policy_name]

    np = template_network_policy
    np.name = network_policy_name

    test_network_policy = network_policies.create(np)

    fetch_handle = test_network_policy.fetch()
    assert fetch_handle.name.upper() == network_policy_name.upper()

    network_policy_handle.drop()

    with pytest.raises(NotFoundError):
        test_network_policy.fetch()

    with pytest.raises(NotFoundError):
        test_network_policy.drop(if_exists=False)

    test_network_policy.drop(if_exists=True)



import copy

import pytest

from tests.integ.utils import random_string

from .conftest import test_generic_string_secret_template


@pytest.fixture(scope="module")
def secrets_extended(secrets):
    names_list = []

    for _ in range(5):
        names_list.append(random_string(10, "test_secret_iter_a_"))

    for _ in range(7):
        names_list.append(random_string(10, "test_secret_iter_b_"))

    for _ in range(3):
        names_list.append(random_string(10, "test_secret_iter_c_"))

    try:
        for name in names_list:
            secret = copy.deepcopy(test_generic_string_secret_template)
            secret.name = name
            secrets.create(secret)

        yield secrets
    finally:
        for name in names_list:
            secrets[name].drop(if_exists=True)


def test_iter_raw(secrets_extended):
    assert len(list(secrets_extended.iter())) >= 15


def test_iter_like(secrets_extended):
    assert len(list(secrets_extended.iter(like="test_secret_iter_a_%"))) == 5
    assert len(list(secrets_extended.iter(like="test_secret_iter_b_%"))) == 7
    assert len(list(secrets_extended.iter(like="test_secret_iter_c_%"))) == 3
    assert len(list(secrets_extended.iter(like="TEST_SECRET_ITER_C_%"))) == 3
    assert len(list(secrets_extended.iter(like="nonexistent_pattern_%"))) == 0


def test_iter_limit(secrets_extended):
    assert len(list(secrets_extended.iter(limit=2))) == 2


def test_iter_starts_with(secrets_extended):
    assert len(list(secrets_extended.iter(starts_with="test_secret_iter_a_".upper()))) == 5
    assert len(list(secrets_extended.iter(starts_with="test_secret_iter_a_"))) == 0
    assert len(list(secrets_extended.iter(starts_with="test_secret_iter_d_".upper()))) == 0


def test_iter_from_name(secrets_extended):
    # The limit keyword is required for the from keyword to function, limit=20 was chosen arbitrarily
    # as it does not affect the test
    secrets_from_b = list(secrets_extended.iter(limit=20, from_name="test_secret_iter_b_".upper()))
    assert len(secrets_from_b) == 10



"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class ServiceRole(BaseModel):
    """A model object representing the ServiceRole resource.

    Constructs an object of type ServiceRole with the provided properties.

    Parameters
    __________
    created_on : datetime, optional
        Date and time when the service role was created — **Read-only:** *any user-provided value will be ignored.*
    name : str, optional
        Service role name — **Read-only:** *any user-provided value will be ignored.*
    comment : str, optional
        Comment, if any, for the service role — **Read-only:** *any user-provided value will be ignored.*
    """

    created_on: Optional[datetime] = None

    name: Optional[StrictStr] = None

    comment: Optional[StrictStr] = None

    __properties = ["created_on", "name", "comment"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ServiceRole:
        """Create an instance of ServiceRole from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "name",
                    "comment",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ServiceRole:
        """Create an instance of ServiceRole from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ServiceRole.model_validate(obj)

        _obj = ServiceRole.model_validate(
            {
                "created_on": obj.get("created_on"),
                "name": obj.get("name"),
                "comment": obj.get("comment"),
            }
        )

        return _obj


class ServiceRoleModel:
    def __init__(
        self,  # optional properties
        created_on: Optional[datetime] = None,
        name: Optional[str] = None,
        comment: Optional[str] = None,
    ):
        """A model object representing the ServiceRole resource.

        Constructs an object of type ServiceRole with the provided properties.

        Parameters
        __________
        created_on : datetime, optional
            Date and time when the service role was created
        name : str, optional
            Service role name
        comment : str, optional
            Comment, if any, for the service role
        """
        self.created_on = created_on
        self.name = name
        self.comment = comment

    __properties = ["created_on", "name", "comment"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ServiceRole(
            created_on=self.created_on,
            name=self.name,
            comment=self.comment,
        )

    @classmethod
    def _from_model(cls, model) -> ServiceRoleModel:
        return ServiceRoleModel(
            created_on=model.created_on,
            name=model.name,
            comment=model.comment,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ServiceRole.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ServiceRole object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ServiceRoleModel:
        """Create an instance of ServiceRole from a dict.

        This method constructs a ServiceRole object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ServiceRole
            A ServiceRole object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ServiceRole.from_dict(obj))


ServiceRole._model_class = ServiceRoleModel



import pytest


@pytest.fixture()
def anaconda_package_available(cursor, warehouse, backup_warehouse_fixture):
    del backup_warehouse_fixture
    packages_required = ["snowflake-snowpark-python", "atpublic"]
    required_available = True
    cursor.execute(f"use warehouse {warehouse.name}").fetchone()
    for package in packages_required:
        packages = cursor.execute(
            f"select * from information_schema.packages where package_name = '{package}';"
        ).fetchmany(3)
        required_available &= packages is not None and len(packages) > 0

    if not required_available:
        pytest.xfail(reason="Anaconda packages not available")


@pytest.fixture()
def maven_snowpark_jar_available(cursor, warehouse, backup_warehouse_fixture):
    del backup_warehouse_fixture
    packages_required = ["com.snowflake:snowpark"]
    required_available = True
    cursor.execute(f"use warehouse {warehouse.name}").fetchone()
    for package in packages_required:
        packages = cursor.execute(
            f"select * from information_schema.packages where package_name = '{package}';"
        ).fetchmany(3)
        required_available &= packages is not None and len(packages) > 0

    if not required_available:
        pytest.xfail(reason="Maven Snowpark jar not available")


@pytest.fixture(scope="session")
def shared_database_available(cursor):
    shared_database = cursor.execute("SHOW SHARES LIKE 'SAMPLE_DATA';").fetchone()
    if shared_database is None or len(shared_database) == 0 or shared_database[2] != "SFSALESSHARED.SFC_SAMPLES_PROD3":
        pytest.xfail(reason="There's no shared database")


@pytest.fixture(scope="session")
def skip_for_snowflake_account(cursor):
    account_name = cursor.execute("SELECT CURRENT_ACCOUNT();").fetchone()
    if len(account_name) > 0 and "SNOWFLAKE" == account_name[0]:
        pytest.skip("Skip when running in Snowflake account")


@pytest.fixture
def qa_mode_enabled(cursor):
    alter_prefix = "alter session "
    try:
        cursor.execute(alter_prefix + "set QA_MODE = true").fetchone()
        yield
        cursor.execute(alter_prefix + "unset QA_MODE").fetchone()
    except Exception:
        pytest.xfail("QA Mode is not available")


@pytest.fixture(scope="session")
def my_integration_exists(cursor):
    integrations = cursor.execute("SHOW INTEGRATIONS LIKE 'my_integration';").fetchone()
    if integrations is None or len(integrations) == 0 or integrations[1] != "my_integration":
        pytest.xfail(reason="my_integration does not exist")



"""Manages Snowflake Procedures."""

from ..procedure._generated.models import (
    Argument,
    CallArgument,
    CallArgumentList,
    ColumnType,
    ErrorResponse,
    JavaFunction,
    JavaScriptFunction,
    Procedure,
    PythonFunction,
    ReturnDataType,
    ReturnTable,
    ReturnType,
    ScalaFunction,
    SQLFunction,
)
from ._procedure import ProcedureCollection, ProcedureResource


__all__ = [
    "Argument",
    "CallArgument",
    "CallArgumentList",
    "ColumnType",
    "ErrorResponse",
    "JavaFunction",
    "JavaScriptFunction",
    "Procedure",
    "ProcedureCollection",
    "ProcedureResource",
    "PythonFunction",
    "ReturnDataType",
    "ReturnTable",
    "ReturnType",
    "ScalaFunction",
    "SQLFunction",
]



# coding: utf-8

# flake8: noqa

"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.service._generated.api.service_api import ServiceApi

# import models into sdk package
from snowflake.core.service._generated.models.error_response import ErrorResponse
from snowflake.core.service._generated.models.fetch_service_logs200_response import FetchServiceLogs200Response
from snowflake.core.service._generated.models.fetch_service_status200_response import FetchServiceStatus200Response
from snowflake.core.service._generated.models.grant_of import GrantOf
from snowflake.core.service._generated.models.job_service import JobService
from snowflake.core.service._generated.models.service import Service
from snowflake.core.service._generated.models.service_container import ServiceContainer
from snowflake.core.service._generated.models.service_endpoint import ServiceEndpoint
from snowflake.core.service._generated.models.service_instance import ServiceInstance
from snowflake.core.service._generated.models.service_role import ServiceRole
from snowflake.core.service._generated.models.service_role_grant_to import ServiceRoleGrantTo
from snowflake.core.service._generated.models.service_spec import ServiceSpec
from snowflake.core.service._generated.models.service_spec_inline_text import ServiceSpecInlineText
from snowflake.core.service._generated.models.service_spec_stage_file import ServiceSpecStageFile
from snowflake.core.service._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.service._generated.models.success_response import SuccessResponse



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.password_policy._generated.api.password_policy_api import PasswordPolicyApi

__all__ = [
    "PasswordPolicyApi",
]



from typing import TYPE_CHECKING

from ._generated.api.network_policy_api_base import NetworkPolicyCollectionBase, NetworkPolicyResourceBase


if TYPE_CHECKING:
    from snowflake.core._root import Root


class NetworkPolicyCollection(NetworkPolicyCollectionBase):
    """Represents the collection operations on the Snowflake Network Policy resource.

    With this collection, you can create, iterate through, and fetch network policies
    that you have access to in the current context.

    Examples
    ________
    Creating a network policy instance with only a single ip allowed:

    >>> network_policies = root.network_policies
    >>> new_network_policy = NetworkPolicy(
    ...     name="single_ip_policy", allowed_ip_list=["192.168.1.32/32"], blocked_ip_list=["0.0.0.0"]
    ... )
    >>> network_policies.create(new_network_policy)
    """

    def __init__(self, root: "Root"):
        super().__init__(root, NetworkPolicyResource)


class NetworkPolicyResource(NetworkPolicyResourceBase):
    """Represents a reference to a Snowflake Network Policy resource.

    With this network policy reference, you can fetch information about a network policy, as well
    as perform certain actions on it.
    """

    _plural_name = "network_policies"



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.confidence import Confidence, ConfidenceModel
from snowflake.core.cortex.analyst_service._generated.models.message_content import MessageContent


class MessageContentSqlObject(MessageContent):
    """A model object representing the MessageContentSqlObject resource.

    Constructs an object of type MessageContentSqlObject with the provided properties.

    Parameters
    __________
    statement : str
        The executable SQL statement
    confidence : Confidence, optional
    """

    statement: StrictStr

    confidence: Optional[Confidence] = None

    __properties = ["type", "statement", "confidence"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MessageContentSqlObject:
        """Create an instance of MessageContentSqlObject from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of confidence
        if self.confidence:
            _dict["confidence"] = self.confidence.to_dict()

        _dict["type"] = MessageContent.get_child_model_discriminator_value("MessageContentSqlObject")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentSqlObject:
        """Create an instance of MessageContentSqlObject from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return MessageContentSqlObject.model_validate(obj)

        _obj = MessageContentSqlObject.model_validate(
            {
                "statement": obj.get("statement"),
                "confidence": Confidence.from_dict(obj.get("confidence"))
                if obj.get("confidence") is not None
                else None,
            }
        )

        return _obj


class MessageContentSqlObjectModel(MessageContent):
    def __init__(
        self,
        statement: str,
        # optional properties
        confidence: Optional[Confidence] = None,
    ):
        """A model object representing the MessageContentSqlObject resource.

        Constructs an object of type MessageContentSqlObject with the provided properties.

        Parameters
        __________
        statement : str
            The executable SQL statement
        confidence : Confidence, optional
        """
        super().__init__()
        self.statement = statement
        self.confidence = confidence

    __properties = ["type", "statement", "confidence"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return MessageContentSqlObject(
            statement=self.statement,
            confidence=self.confidence._to_model() if self.confidence is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> MessageContentSqlObjectModel:
        return MessageContentSqlObjectModel(
            statement=model.statement,
            confidence=ConfidenceModel._from_model(model.confidence) if model.confidence else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a MessageContentSqlObject.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the MessageContentSqlObject object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentSqlObjectModel:
        """Create an instance of MessageContentSqlObject from a dict.

        This method constructs a MessageContentSqlObject object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        MessageContentSqlObject
            A MessageContentSqlObject object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(MessageContentSqlObject.from_dict(obj))


MessageContentSqlObject._model_class = MessageContentSqlObjectModel



import pytest

from snowflake.core.user_defined_function import PythonFunction, ReturnDataType, UserDefinedFunction
from tests.utils import random_string

from ...utils import ensure_snowflake_version


pytestmark = [pytest.mark.min_sf_ver("8.38.0"), pytest.mark.usefixtures("anaconda_package_available")]


@pytest.fixture(scope="session")
def user_defined_functions_extended(user_defined_functions, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.38.0")

    name_list = []
    for _ in range(5):
        name_list.append(random_string(10, "test_user_defined_function_iter_a_"))
    for _ in range(7):
        name_list.append(random_string(10, "test_user_defined_function_iter_b_"))
    for _ in range(3):
        name_list.append(random_string(10, "test_user_defined_function_iter_c_"))

    for user_defined_function_name in name_list:
        user_defined_functions.create(
            UserDefinedFunction(
                name=user_defined_function_name,
                arguments=[],
                return_type=ReturnDataType(datatype="VARIANT"),
                language_config=PythonFunction(runtime_version="3.13", packages=[], handler="udf"),
                body="""
def udf():
    return {"key": "value"}
            """,
            )
        )

    try:
        yield user_defined_functions
    finally:
        for user_defined_function_name in name_list:
            user_defined_functions[f"{user_defined_function_name}()"].drop()


def test_iter_raw(user_defined_functions_extended):
    assert len(list(user_defined_functions_extended.iter())) >= 15


def test_iter_like(user_defined_functions_extended):
    assert len(list(user_defined_functions_extended.iter(like="test_user_defined_function_iter_"))) == 0
    assert len(list(user_defined_functions_extended.iter(like="test_user_defined_function_iter_a_%%"))) == 5
    assert len(list(user_defined_functions_extended.iter(like="test_user_defined_function_iter_b_%%"))) == 7
    assert len(list(user_defined_functions_extended.iter(like="test_user_defined_function_iter_c_%%"))) == 3



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.notification_integration import (
    NotificationEmail,
    NotificationIntegration,
    NotificationIntegrationCollection,
    NotificationIntegrationResource,
)
from snowflake.core.notification_integration._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
NOTIFICATION_INTEGRATION = NotificationIntegration(name="my_int", notification_hook=NotificationEmail())


@pytest.fixture
def notification_integrations(fake_root):
    return NotificationIntegrationCollection(fake_root)


@pytest.fixture
def notification_integration(notification_integrations):
    return notification_integrations["my_int"]


def test_create_notification_integration(fake_root, notification_integrations):
    args = (fake_root, "POST", BASE_URL + "/notification-integrations")
    kwargs = extra_params(query_params=[], body={"name": "my_int", "notification_hook": {"type": "EMAIL"}})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        ni_res = notification_integrations.create(NOTIFICATION_INTEGRATION)
        assert isinstance(ni_res, NotificationIntegrationResource)
        assert ni_res.name == "my_int"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = notification_integrations.create_async(NOTIFICATION_INTEGRATION)
        assert isinstance(op, PollingOperation)
        ni_res = op.result()
        assert ni_res.name == "my_int"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_notification_integration(fake_root, notification_integrations):
    args = (fake_root, "GET", BASE_URL + "/notification-integrations")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        notification_integrations.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = notification_integrations.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_notification_integration(fake_root, notification_integration):
    from snowflake.core.notification_integration._generated.models import NotificationEmail as NotificationEmailModel
    from snowflake.core.notification_integration._generated.models import (
        NotificationIntegration as NotificationIntegrationModel,
    )

    model = NotificationIntegrationModel(name="my_int", notification_hook=NotificationEmailModel())
    args = (fake_root, "GET", BASE_URL + "/notification-integrations/my_int")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        notification_integration.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = notification_integration.fetch_async()
        assert isinstance(op, PollingOperation)
        ni = op.result()
        assert ni.to_dict() == NOTIFICATION_INTEGRATION.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_notification_integration(fake_root, notification_integration):
    args = (fake_root, "DELETE", BASE_URL + "/notification-integrations/my_int")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        notification_integration.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = notification_integration.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, notification_integration, tag):
    args = (fake_root, "POST", BASE_URL + "/notification-integrations/my_int:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        notification_integration.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = notification_integration.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, notification_integration, tag):
    args = (fake_root, "POST", BASE_URL + "/notification-integrations/my_int:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        notification_integration.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = notification_integration.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, notification_integration):
    args = (fake_root, "GET", BASE_URL + "/notification-integrations/my_int:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert notification_integration.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = notification_integration.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.confidence import Confidence, ConfidenceModel
from snowflake.core.cortex.analyst_service._generated.models.message_content_delta import (
    MessageContentDelta,
)


class MessageContentDeltaSqlObject(MessageContentDelta):
    """A model object representing the MessageContentDeltaSqlObject resource.

    Constructs an object of type MessageContentDeltaSqlObject with the provided properties.

    Parameters
    __________
    statement_delta : str
        The delta of the sql statement, clients should concatenate all deltas for the same index

    index : int, optional
        The index of the content array this delta object represents

    confidence : Confidence, optional
    """

    statement_delta: StrictStr

    confidence: Optional[Confidence] = None

    __properties = ["type", "index", "statement_delta", "confidence"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MessageContentDeltaSqlObject:
        """Create an instance of MessageContentDeltaSqlObject from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of confidence
        if self.confidence:
            _dict["confidence"] = self.confidence.to_dict()

        _dict["type"] = MessageContentDelta.get_child_model_discriminator_value("MessageContentDeltaSqlObject")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentDeltaSqlObject:
        """Create an instance of MessageContentDeltaSqlObject from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return MessageContentDeltaSqlObject.model_validate(obj)

        _obj = MessageContentDeltaSqlObject.model_validate(
            {
                "index": obj.get("index"),
                "statement_delta": obj.get("statement_delta"),
                "confidence": Confidence.from_dict(obj.get("confidence"))
                if obj.get("confidence") is not None
                else None,
            }
        )

        return _obj


class MessageContentDeltaSqlObjectModel(MessageContentDelta):
    def __init__(
        self,
        statement_delta: str,
        # optional properties
        index: Optional[int] = None,
        confidence: Optional[Confidence] = None,
    ):
        """A model object representing the MessageContentDeltaSqlObject resource.

        Constructs an object of type MessageContentDeltaSqlObject with the provided properties.

        Parameters
        __________
        statement_delta : str
            The delta of the sql statement, clients should concatenate all deltas for the same index

        index : int, optional
            The index of the content array this delta object represents

        confidence : Confidence, optional
        """
        super().__init__(
            index=index,
        )
        self.statement_delta = statement_delta
        self.confidence = confidence

    __properties = ["type", "index", "statement_delta", "confidence"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return MessageContentDeltaSqlObject(
            index=self.index,
            statement_delta=self.statement_delta,
            confidence=self.confidence._to_model() if self.confidence is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> MessageContentDeltaSqlObjectModel:
        return MessageContentDeltaSqlObjectModel(
            index=model.index,
            statement_delta=model.statement_delta,
            confidence=ConfidenceModel._from_model(model.confidence) if model.confidence else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a MessageContentDeltaSqlObject.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the MessageContentDeltaSqlObject object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentDeltaSqlObjectModel:
        """Create an instance of MessageContentDeltaSqlObject from a dict.

        This method constructs a MessageContentDeltaSqlObject object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        MessageContentDeltaSqlObject
            A MessageContentDeltaSqlObject object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(MessageContentDeltaSqlObject.from_dict(obj))


MessageContentDeltaSqlObject._model_class = MessageContentDeltaSqlObjectModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.network_rule._generated.api.network_rule_api import NetworkRuleApi

__all__ = [
    "NetworkRuleApi",
]



from __future__ import annotations


class Securable:
    """Class to represent an snowflake entity that is being secured by a :class:`Privileges`."""

    def __init__(self, name: str, securable_type: str, scope: Securable | None = None) -> None:
        self._name = name
        self._securable_type = securable_type
        self._scope = scope

    @property
    def name(self) -> str:
        """String that specifies the name of resource that is being secured by a privilege."""
        return self._name

    @property
    def securable_type(self) -> str:
        """String that specifies the type of resource that is being secured by a privilege."""
        return self._securable_type

    @property
    def scope(self) -> Securable | None:
        """This is the scope of the resource that is being secured by a privilege."""
        return self._scope


class Securables:
    """
    Util Class with static method to create various ``Securable`` class instances.

    Examples
    ________
    Creating a Securable:

    >>> Securables.account("test-account")
    >>> Securables.database("testdb")
    >>> Securables.current_account
    """

    current_account = Securable("-", "account")

    @staticmethod
    def database(name: str) -> Securable:
        """Securable for a database."""
        return Securable(name, "database")

    @staticmethod
    def database_role(name: str) -> Securable:
        """Securable for a database role."""
        return Securable(name, "database role")

    @staticmethod
    def application_role(name: str) -> Securable:
        """Securable for a application role."""
        return Securable(name, "application role")

    @staticmethod
    def service_role(name: str) -> Securable:
        """Securable for a service role."""
        return Securable(name, "service role")

    @staticmethod
    def role(name: str) -> Securable:
        """Securable for a role."""
        return Securable(name, "role")

    @staticmethod
    def account(name: str) -> Securable:
        """Securable for an account."""
        return Securable(name, "account")

    @staticmethod
    def schema(name: str) -> Securable:
        """Securable for a schema."""
        return Securable(name, "schema")

    @staticmethod
    def integration(name: str) -> Securable:
        """Securable for an integration."""
        return Securable(name, "integration")

    @staticmethod
    def user(name: str) -> Securable:
        """Securable for a user."""
        return Securable(name, "user")

    @staticmethod
    def resource_monitor(name: str) -> Securable:
        """Securable for a resource monitor."""
        return Securable(name, "RESOURCE MONITOR")

    @staticmethod
    def warehouse(name: str) -> Securable:
        """Securable for a warehouse."""
        return Securable(name, "WAREHOUSE")

    @staticmethod
    def compute_pool(name: str) -> Securable:
        """Securable for a compute pool."""
        return Securable(name, "COMPUTE POOL")

    @staticmethod
    def connection(name: str) -> Securable:
        """Securable for a connection."""
        return Securable(name, "CONNECTION")

    @staticmethod
    def failover_group(name: str) -> Securable:
        """Securable for a failover group."""
        return Securable(name, "FAILOVER GROUP")

    @staticmethod
    def replication_group(name: str) -> Securable:
        """Securable for a replication group."""
        return Securable(name, "REPLICATION GROUP")

    @staticmethod
    def external_volume(name: str) -> Securable:
        """Securable for an external volume."""
        return Securable(name, "EXTERNAL VOLUME")

    @staticmethod
    def alert(name: str) -> Securable:
        """Securable for an alert."""
        return Securable(name, "ALERT")

    @staticmethod
    def authentication_policy(name: str) -> Securable:
        """Securable for an authentication policy."""
        return Securable(name, "AUTHENTICATION POLICY")

    @staticmethod
    def data_metric_function(name: str) -> Securable:
        """Securable for a data metric function."""
        return Securable(name, "DATA METRIC FUNCTION")

    @staticmethod
    def dynamic_table(name: str) -> Securable:
        """Securable for a dynamic table."""
        return Securable(name, "DYNAMIC TABLE")

    @staticmethod
    def event_table(name: str) -> Securable:
        """Securable for an event table."""
        return Securable(name, "EVENT TABLE")

    @staticmethod
    def external_table(name: str) -> Securable:
        """Securable for an external table."""
        return Securable(name, "EXTERNAL TABLE")

    @staticmethod
    def file_format(name: str) -> Securable:
        """Securable for a file format."""
        return Securable(name, "FILE FORMAT")

    @staticmethod
    def function(name: str) -> Securable:
        """Securable for a function."""
        return Securable(name, "FUNCTION")

    @staticmethod
    def git_repository(name: str) -> Securable:
        """Securable for a git repository."""
        return Securable(name, "GIT REPOSITORY")

    @staticmethod
    def hybrid_table(name: str) -> Securable:
        """Securable for a hybrid table."""
        return Securable(name, "HYBRID TABLE")

    @staticmethod
    def image_repository(name: str) -> Securable:
        """Securable for an image repository."""
        return Securable(name, "IMAGE REPOSITORY")

    @staticmethod
    def iceberg_table(name: str) -> Securable:
        """Securable for an iceberg table."""
        return Securable(name, "ICEBERG TABLE")

    @staticmethod
    def masking_policy(name: str) -> Securable:
        """Securable for a masking policy."""
        return Securable(name, "MASKING POLICY")

    @staticmethod
    def materialized_view(name: str) -> Securable:
        """Securable for a materialized view."""
        return Securable(name, "MATERIALIZED VIEW")

    @staticmethod
    def model(name: str) -> Securable:
        """Securable for a model."""
        return Securable(name, "MODEL")

    @staticmethod
    def network_rule(name: str) -> Securable:
        """Securable for a network rule."""
        return Securable(name, "NETWORK RULE")

    @staticmethod
    def packages_policy(name: str) -> Securable:
        """Securable for a packages policy."""
        return Securable(name, "PACKAGES POLICY")

    @staticmethod
    def password_policy(name: str) -> Securable:
        """Securable for a password policy."""
        return Securable(name, "PASSWORD POLICY")

    @staticmethod
    def pipe(name: str) -> Securable:
        """Securable for a pipe."""
        return Securable(name, "PIPE")

    @staticmethod
    def procedure(name: str) -> Securable:
        """Securable for a procedure."""
        return Securable(name, "PROCEDURE")

    @staticmethod
    def projection_policy(name: str) -> Securable:
        """Securable for a projection policy."""
        return Securable(name, "PROJECTION POLICY")

    @staticmethod
    def row_access_policy(name: str) -> Securable:
        """Securable for a row access policy."""
        return Securable(name, "ROW ACCESS POLICY")

    @staticmethod
    def secret(name: str) -> Securable:
        """Securable for a secret."""
        return Securable(name, "SECRET")

    @staticmethod
    def service(name: str) -> Securable:
        """Securable for a service."""
        return Securable(name, "SERVICE")

    @staticmethod
    def session_policy(name: str) -> Securable:
        """Securable for a session policy."""
        return Securable(name, "SESSION POLICY")

    @staticmethod
    def sequence(name: str) -> Securable:
        """Securable for a sequence."""
        return Securable(name, "SEQUENCE")

    @staticmethod
    def stage(name: str) -> Securable:
        """Securable for a stage."""
        return Securable(name, "STAGE")

    @staticmethod
    def stream(name: str) -> Securable:
        """Securable for a stream."""
        return Securable(name, "STREAM")

    @staticmethod
    def table(name: str) -> Securable:
        """Securable for a table."""
        return Securable(name, "TABLE")

    @staticmethod
    def tag(name: str) -> Securable:
        """Securable for a tag."""
        return Securable(name, "TAG")

    @staticmethod
    def task(name: str) -> Securable:
        """Securable for a task."""
        return Securable(name, "TASK")

    @staticmethod
    def view(name: str) -> Securable:
        """Securable for a view."""
        return Securable(name, "VIEW")

    @staticmethod
    def streamlit(name: str) -> Securable:
        """Securable for a streamlit."""
        return Securable(name, "STREAMLIT")

    @staticmethod
    def notebook(name: str) -> Securable:
        """Securable for a notebook."""
        return Securable(name, "NOTEBOOK")

    @staticmethod
    def all_aggregation_policies(scope: Securable) -> Securable:
        """Securable for all aggregation policies."""
        return Securable("ALL", "AGGREGATION POLICIES", scope)

    @staticmethod
    def all_alerts(scope: Securable) -> Securable:
        """Securable for all alerts."""
        return Securable("ALL", "ALERTS", scope)

    @staticmethod
    def all_authentication_policies(scope: Securable) -> Securable:
        """Securable for all authentication policies."""
        return Securable("ALL", "AUTHENTICATION POLICIES", scope)

    @staticmethod
    def all_data_metric_functions(scope: Securable) -> Securable:
        """Securable for all data metric functions."""
        return Securable("ALL", "DATA METRIC FUNCTIONS", scope)

    @staticmethod
    def all_dynamic_tables(scope: Securable) -> Securable:
        """Securable for all dynamic tables."""
        return Securable("ALL", "DYNAMIC TABLES", scope)

    @staticmethod
    def all_event_tables(scope: Securable) -> Securable:
        """Securable for all event tables."""
        return Securable("ALL", "EVENT TABLES", scope)

    @staticmethod
    def all_external_tables(scope: Securable) -> Securable:
        """Securable for all external tables."""
        return Securable("ALL", "EXTERNAL TABLES", scope)

    @staticmethod
    def all_file_formats(scope: Securable) -> Securable:
        """Securable for all file formats."""
        return Securable("ALL", "FILE FORMATS", scope)

    @staticmethod
    def all_functions(scope: Securable) -> Securable:
        """Securable for all functions."""
        return Securable("ALL", "FUNCTIONS", scope)

    @staticmethod
    def all_git_repositories(scope: Securable) -> Securable:
        """Securable for all git repositories."""
        return Securable("ALL", "GIT REPOSITORIES", scope)

    @staticmethod
    def all_hybrid_tables(scope: Securable) -> Securable:
        """Securable for all hybrid tables."""
        return Securable("ALL", "HYBRID TABLES", scope)

    @staticmethod
    def all_image_repositories(scope: Securable) -> Securable:
        """Securable for all image repositories."""
        return Securable("ALL", "IMAGE REPOSITORIES", scope)

    @staticmethod
    def all_iceberg_tables(scope: Securable) -> Securable:
        """Securable for all iceberg tables."""
        return Securable("ALL", "ICEBERG TABLES", scope)

    @staticmethod
    def all_masking_policies(scope: Securable) -> Securable:
        """Securable for all masking policies."""
        return Securable("ALL", "MASKING POLICIES", scope)

    @staticmethod
    def all_materialized_views(scope: Securable) -> Securable:
        """Securable for all materialized views."""
        return Securable("ALL", "MATERIALIZED VIEWS", scope)

    @staticmethod
    def all_models(scope: Securable) -> Securable:
        """Securable for all models."""
        return Securable("ALL", "MODELS", scope)

    @staticmethod
    def all_network_rules(scope: Securable) -> Securable:
        """Securable for all network rules."""
        return Securable("ALL", "NETWORK RULES", scope)

    @staticmethod
    def all_packages_policies(scope: Securable) -> Securable:
        """Securable for all packages policies."""
        return Securable("ALL", "PACKAGES POLICIES", scope)

    @staticmethod
    def all_password_policies(scope: Securable) -> Securable:
        """Securable for all password policies."""
        return Securable("ALL", "PASSWORD POLICIES", scope)

    @staticmethod
    def all_procedures(scope: Securable) -> Securable:
        """Securable for all procedures."""
        return Securable("ALL", "PROCEDURES", scope)

    @staticmethod
    def all_projection_policies(scope: Securable) -> Securable:
        """Securable for all projection policies."""
        return Securable("ALL", "PROJECTION POLICIES", scope)

    @staticmethod
    def all_row_access_policies(scope: Securable) -> Securable:
        """Securable for all row access policies."""
        return Securable("ALL", "ROW ACCESS POLICIES", scope)

    @staticmethod
    def all_schemas(scope: Securable) -> Securable:
        """Securable for all schemas."""
        return Securable("ALL", "SCHEMAS", scope)

    @staticmethod
    def all_secrets(scope: Securable) -> Securable:
        """Securable for all secrets."""
        return Securable("ALL", "SECRETS", scope)

    @staticmethod
    def all_services(scope: Securable) -> Securable:
        """Securable for all services."""
        return Securable("ALL", "SERVICES", scope)

    @staticmethod
    def all_session_policies(scope: Securable) -> Securable:
        """Securable for all session policies."""
        return Securable("ALL", "SESSION POLICIES", scope)

    @staticmethod
    def all_sequences(scope: Securable) -> Securable:
        """Securable for all sequences."""
        return Securable("ALL", "SEQUENCES", scope)

    @staticmethod
    def all_stages(scope: Securable) -> Securable:
        """Securable for all stages."""
        return Securable("ALL", "STAGES", scope)

    @staticmethod
    def all_streams(scope: Securable) -> Securable:
        """Securable for all streams."""
        return Securable("ALL", "STREAMS", scope)

    @staticmethod
    def all_tables(scope: Securable) -> Securable:
        """Securable for all tables."""
        return Securable("ALL", "TABLES", scope)

    @staticmethod
    def all_tags(scope: Securable) -> Securable:
        """Securable for all tags."""
        return Securable("ALL", "TAGS", scope)

    @staticmethod
    def all_tasks(scope: Securable) -> Securable:
        """Securable for all tasks."""
        return Securable("ALL", "TASKS", scope)

    @staticmethod
    def all_views(scope: Securable) -> Securable:
        """Securable for all views."""
        return Securable("ALL", "VIEWS", scope)

    @staticmethod
    def all_streamlits(scope: Securable) -> Securable:
        """Securable for all streamlits."""
        return Securable("ALL", "STREAMLITS", scope)

    @staticmethod
    def all_notebooks(scope: Securable) -> Securable:
        """Securable for all notebooks."""
        return Securable("ALL", "NOTEBOOKS", scope)

    @staticmethod
    def future_aggregation_policies(scope: Securable) -> Securable:
        """Securable for future aggregation policies."""
        return Securable("FUTURE", "AGGREGATION POLICIES", scope)

    @staticmethod
    def future_alerts(scope: Securable) -> Securable:
        """Securable for future alerts."""
        return Securable("FUTURE", "ALERTS", scope)

    @staticmethod
    def future_authentication_policies(scope: Securable) -> Securable:
        """Securable for future authentication policies."""
        return Securable("FUTURE", "AUTHENTICATION POLICIES", scope)

    @staticmethod
    def future_data_metric_functions(scope: Securable) -> Securable:
        """Securable for future data metric functions."""
        return Securable("FUTURE", "DATA METRIC FUNCTIONS", scope)

    @staticmethod
    def future_dynamic_tables(scope: Securable) -> Securable:
        """Securable for future dynamic tables."""
        return Securable("FUTURE", "DYNAMIC TABLES", scope)

    @staticmethod
    def future_event_tables(scope: Securable) -> Securable:
        """Securable for future event tables."""
        return Securable("FUTURE", "EVENT TABLES", scope)

    @staticmethod
    def future_external_tables(scope: Securable) -> Securable:
        """Securable for future external tables."""
        return Securable("FUTURE", "EXTERNAL TABLES", scope)

    @staticmethod
    def future_file_formats(scope: Securable) -> Securable:
        """Securable for future file formats."""
        return Securable("FUTURE", "FILE FORMATS", scope)

    @staticmethod
    def future_functions(scope: Securable) -> Securable:
        """Securable for future functions."""
        return Securable("FUTURE", "FUNCTIONS", scope)

    @staticmethod
    def future_git_repositories(scope: Securable) -> Securable:
        """Securable for future git repositories."""
        return Securable("FUTURE", "GIT REPOSITORIES", scope)

    @staticmethod
    def future_hybrid_tables(scope: Securable) -> Securable:
        """Securable for future hybrid tables."""
        return Securable("FUTURE", "HYBRID TABLES", scope)

    @staticmethod
    def future_image_repositories(scope: Securable) -> Securable:
        """Securable for future image repositories."""
        return Securable("FUTURE", "IMAGE REPOSITORIES", scope)

    @staticmethod
    def future_iceberg_tables(scope: Securable) -> Securable:
        """Securable for future iceberg tables."""
        return Securable("FUTURE", "ICEBERG TABLES", scope)

    @staticmethod
    def future_masking_policies(scope: Securable) -> Securable:
        """Securable for future masking policies."""
        return Securable("FUTURE", "MASKING POLICIES", scope)

    @staticmethod
    def future_materialized_views(scope: Securable) -> Securable:
        """Securable for future materialized views."""
        return Securable("FUTURE", "MATERIALIZED VIEWS", scope)

    @staticmethod
    def future_models(scope: Securable) -> Securable:
        """Securable for future models."""
        return Securable("FUTURE", "MODELS", scope)

    @staticmethod
    def future_network_rules(scope: Securable) -> Securable:
        """Securable for future network rules."""
        return Securable("FUTURE", "NETWORK RULES", scope)

    @staticmethod
    def future_packages_policies(scope: Securable) -> Securable:
        """Securable for future packages policies."""
        return Securable("FUTURE", "PACKAGES POLICIES", scope)

    @staticmethod
    def future_password_policies(scope: Securable) -> Securable:
        """Securable for future password policies."""
        return Securable("FUTURE", "PASSWORD POLICIES", scope)

    @staticmethod
    def future_procedures(scope: Securable) -> Securable:
        """Securable for future procedures."""
        return Securable("FUTURE", "PROCEDURES", scope)

    @staticmethod
    def future_projection_policies(scope: Securable) -> Securable:
        """Securable for future projection policies."""
        return Securable("FUTURE", "PROJECTION POLICIES", scope)

    @staticmethod
    def future_row_access_policies(scope: Securable) -> Securable:
        """Securable for future row access policies."""
        return Securable("FUTURE", "ROW ACCESS POLICIES", scope)

    @staticmethod
    def future_schemas(scope: Securable) -> Securable:
        """Securable for future schemas."""
        return Securable("FUTURE", "SCHEMAS", scope)

    @staticmethod
    def future_secrets(scope: Securable) -> Securable:
        """Securable for future secrets."""
        return Securable("FUTURE", "SECRETS", scope)

    @staticmethod
    def future_services(scope: Securable) -> Securable:
        """Securable for future services."""
        return Securable("FUTURE", "SERVICES", scope)

    @staticmethod
    def future_session_policies(scope: Securable) -> Securable:
        """Securable for future session policies."""
        return Securable("FUTURE", "SESSION POLICIES", scope)

    @staticmethod
    def future_sequences(scope: Securable) -> Securable:
        """Securable for future sequences."""
        return Securable("FUTURE", "SEQUENCES", scope)

    @staticmethod
    def future_stages(scope: Securable) -> Securable:
        """Securable for future stages."""
        return Securable("FUTURE", "STAGES", scope)

    @staticmethod
    def future_streams(scope: Securable) -> Securable:
        """Securable for future streams."""
        return Securable("FUTURE", "STREAMS", scope)

    @staticmethod
    def future_tables(scope: Securable) -> Securable:
        """Securable for future tables."""
        return Securable("FUTURE", "TABLES", scope)

    @staticmethod
    def future_tags(scope: Securable) -> Securable:
        """Securable for future tags."""
        return Securable("FUTURE", "TAGS", scope)

    @staticmethod
    def future_tasks(scope: Securable) -> Securable:
        """Securable for future tasks."""
        return Securable("FUTURE", "TASKS", scope)

    @staticmethod
    def future_views(scope: Securable) -> Securable:
        """Securable for future views."""
        return Securable("FUTURE", "VIEWS", scope)

    @staticmethod
    def future_streamlits(scope: Securable) -> Securable:
        """Securable for future streamlits."""
        return Securable("FUTURE", "STREAMLITS", scope)

    @staticmethod
    def future_notebooks(scope: Securable) -> Securable:
        """Securable for future notebooks."""
        return Securable("FUTURE", "NOTEBOOKS", scope)



"""Common generated classes shared across all resources."""

from .api_client import ApiClient
from .configuration import Configuration
from .paging import PagedIter
from .rest import RESTClientObject


__all__ = [
    "ApiClient",
    "Configuration",
    "PagedIter",
    "RESTClientObject",
]



import pytest

from snowflake.core.artifact_repository import ArtifactRepository
from tests.integ.fixtures.constants import UUID
from tests.integ.utils import random_string


@pytest.fixture(scope="function")
def setup(artifact_repositories, temp_pypi_api_integration):
    """Set up multiple artifact repositories for iteration testing."""
    names = [random_string(10, f"test_artifact_repository_iter_a_{UUID}_")]
    for _ in range(2):
        names.append(random_string(10, f"test_artifact_repository_iter_b_{UUID}_"))
    for _ in range(3):
        names.append(random_string(10, f"test_artifact_repository_iter_c_{UUID}_"))

    try:
        for name in names:
            ar_def = ArtifactRepository(
                name=name,
                type="PIP",
                api_integration=temp_pypi_api_integration,
                comment="created by test_iter setup",
            )
            artifact_repositories.create(ar_def)
        yield
    finally:
        for name in names:
            artifact_repositories[name].drop(if_exists=True)


def test_iter_artifact_repositories(setup, artifact_repositories):
    """Test iterating over artifact repositories."""
    test_repositories = list(artifact_repositories.iter())
    assert len(test_repositories) == 6

    for ar in test_repositories:
        assert ar.name is not None
        assert ar.type == "PIP"
        assert ar.api_integration is not None


def test_iter_artifact_repositories_with_filter(setup, artifact_repositories):
    """Test iterating over artifact repositories with pattern filter."""
    filter_pattern = f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_%"

    b_pattern_results = list(artifact_repositories.iter(like=filter_pattern))
    assert len(b_pattern_results) == 2

    for ar in b_pattern_results:
        assert f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_" in ar.name


def test_iter_artifact_repositories_empty_filter(setup, artifact_repositories):
    """Test iterating over artifact repositories with filter that matches nothing."""
    no_match_results = list(artifact_repositories.iter(like="nonexistent_pattern_%"))
    assert len(no_match_results) == 0


def test_iter_artifact_repositories_starts_with(setup, artifact_repositories):
    """Test iterating over artifact repositories with starts_with filter."""
    starts_with_pattern = f"TEST_ARTIFACT_REPOSITORY_ITER_C_{UUID}_"

    c_starts_with_results = list(artifact_repositories.iter(starts_with=starts_with_pattern))
    assert len(c_starts_with_results) == 3

    for ar in c_starts_with_results:
        assert ar.name.startswith(starts_with_pattern)


def test_iter_artifact_repositories_with_limit(setup, artifact_repositories):
    """Test iterating over artifact repositories with limit."""
    limited_results = list(artifact_repositories.iter(limit=3))
    assert len(limited_results) == 3

    filter_pattern = f"TEST_ARTIFACT_REPOSITORY_ITER_C_{UUID}_%"
    limited_filtered_results = list(artifact_repositories.iter(like=filter_pattern, limit=2))
    assert len(limited_filtered_results) == 2

    for ar in limited_filtered_results:
        assert f"TEST_ARTIFACT_REPOSITORY_ITER_C_{UUID}_" in ar.name


def test_iter_artifact_repositories_from_name_pagination(setup, artifact_repositories):
    """Test iterating over artifact repositories with from_name pagination parameter."""
    test_results = list(artifact_repositories.iter())

    test_results.sort(key=lambda x: x.name)
    assert len(test_results) == 6

    second_repo_name = test_results[1].name
    from_second_results = list(artifact_repositories.iter(limit=6, from_name=second_repo_name))

    # FROM clause is inclusive, so we expect 5 repos (from index 1 to 5, inclusive)
    assert len(from_second_results) == 5, (
        f"Expected 5 repos from second repo onwards (inclusive), got {len(from_second_results)}"
    )

    for ar in from_second_results:
        assert ar.name >= second_repo_name, f"Expected {ar.name} >= {second_repo_name} (from_name is inclusive)"

    last_repo_name = test_results[-1].name
    from_last_results = list(artifact_repositories.iter(limit=6, from_name=last_repo_name))

    # FROM clause is inclusive, so we expect 1 repo (the last repo itself)
    assert len(from_last_results) == 1, (
        f"Expected 1 repo starting from last repo (inclusive), got {len(from_last_results)}"
    )

    partial_name = f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_"
    from_partial_results = list(artifact_repositories.iter(limit=6, from_name=partial_name))

    # Should get both B repositories (2) + all C repositories (3) = 5 total
    # since partial_name matches the first B repository and from_name is inclusive
    assert len(from_partial_results) == 5, (
        f"Expected 5 repos from partial name match (2 B + 3 C), got {len(from_partial_results)}"
    )

    # Count the B and C repositories separately to verify the partial matching worked
    b_repos = [ar for ar in from_partial_results if f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_" in ar.name]
    c_repos = [ar for ar in from_partial_results if f"TEST_ARTIFACT_REPOSITORY_ITER_C_{UUID}_" in ar.name]

    assert len(b_repos) == 2, f"Expected 2 'B' repos, got {len(b_repos)}"
    assert len(c_repos) == 3, f"Expected 3 'C' repos, got {len(c_repos)}"

    limited_results = list(artifact_repositories.iter(from_name=second_repo_name, limit=2))
    assert len(limited_results) <= 2, "Limited results should respect limit parameter"


def test_iter_artifact_repositories_from_name_with_filter(setup, artifact_repositories):
    """Test from_name pagination combined with like filter."""
    filter_pattern = f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_%"

    all_b_results = list(artifact_repositories.iter(like=filter_pattern))
    assert len(all_b_results) == 2, "Setup should create exactly 2 'B' pattern repositories"

    all_b_results.sort(key=lambda x: x.name)
    from_name_reference = all_b_results[0].name

    from_name_filtered_results = list(
        artifact_repositories.iter(like=filter_pattern, limit=2, from_name=from_name_reference)
    )

    # FROM clause is inclusive, so we expect both B repositories (starting from first one)
    assert len(from_name_filtered_results) == 2, (
        f"Expected 2 'B' repos starting from first (inclusive), got {len(from_name_filtered_results)}"
    )

    for ar in from_name_filtered_results:
        assert f"TEST_ARTIFACT_REPOSITORY_ITER_B_{UUID}_" in ar.name
        assert ar.name >= from_name_reference, f"Expected {ar.name} >= {from_name_reference} (from_name is inclusive)"

    assert from_name_filtered_results[0].name == all_b_results[0].name, "Should get the first 'B' repository"
    assert from_name_filtered_results[1].name == all_b_results[1].name, "Should get the second 'B' repository"



from ._generated import CortexLiteAgentApi
from ._lite_agent_service import AgentRunRequest, CortexAgentService


CortexAgentServiceApi = CortexLiteAgentApi


__all__ = ["CortexAgentService", "AgentRunRequest", "CortexAgentServiceApi"]



from io import BytesIO
from textwrap import dedent
from time import sleep

import pytest

from snowflake.core.exceptions import ConflictError, NotFoundError
from snowflake.core.service import JobService, ServiceCollection, ServiceSpecInlineText, ServiceSpecStageFile
from snowflake.core.service._service import ServiceResource
from tests.utils import random_string


RETRY_TIMES = 5

pytestmark = [pytest.mark.skip_gov]


def wait_for_job_creation(job: ServiceResource):
    # wait for the job to be created if it is an async request otherwise fecth() below might fail if it is executed
    # before the job is created
    for _ in range(RETRY_TIMES):
        try:
            return job.fetch()
        except NotFoundError:
            sleep(5)
    raise Exception(f"Job creation failed: {job.name}")


@pytest.mark.flaky
@pytest.mark.min_sf_ver("8.34")
def test_execute_job_service(services: ServiceCollection, session, imagerepo, shared_compute_pool):
    # create a job from service spec in stage
    job_name = random_string(5, "test_job_")
    stage_name = random_string(5, "test_stage_")
    session.sql(f"create temp stage {stage_name};").collect()
    spec_file = "spec.yaml"
    stage_file = f"@{stage_name}"
    spec = f"{stage_file}/{spec_file}"
    session.file.put_stream(
        BytesIO(
            dedent(f"""\
                spec:
                  containers:
                  - name: hello-world
                    image: {imagerepo}/hello-world:latest
                """).encode()
        ),
        spec,
    )

    test_job_spec = JobService(
        name=job_name,
        compute_pool=shared_compute_pool,
        spec=ServiceSpecStageFile(stage=stage_name, spec_file=spec_file),
    )
    test_job = services.execute_job(test_job_spec)

    fetched_job = wait_for_job_creation(test_job)

    assert fetched_job.name.upper() == job_name.upper()
    assert fetched_job.compute_pool.upper() == shared_compute_pool.upper()
    assert fetched_job.min_instances == 1
    assert fetched_job.max_instances == 1
    assert fetched_job.database_name.upper() == services.database.name.upper()
    assert fetched_job.schema_name.upper() == services.schema.name.upper()

    # create a job with the same name and expect a conflict error
    with pytest.raises(ConflictError):
        services.execute_job(test_job_spec)

    # create a job from inline service spec
    inline_spec = dedent(
        f"""\
        spec:
          containers:
          - name: hello-world
            image: {imagerepo}/hello-world:latest
         """
    )
    job_name = random_string(5, "test_job_")
    test_job_inline_spec = JobService(
        name=job_name, compute_pool=shared_compute_pool, spec=ServiceSpecInlineText(spec_text=inline_spec)
    )
    test_job = services.execute_job(test_job_inline_spec)

    fetched_job_inline_spec = wait_for_job_creation(test_job)

    assert fetched_job_inline_spec.name.upper() == test_job_inline_spec.name.upper()


@pytest.mark.min_sf_ver("8.46")
def test_execute_async_job_service(
    services: ServiceCollection, session, imagerepo, shared_compute_pool, setup_with_connector_execution
):
    test_job = None
    pytest.xfail("Flaky test, fix trace by SNOW-1896405")

    try:
        # create an async job service
        job_name = random_string(5, "test_async_job_")
        stage_name = random_string(5, "test_stage_")
        session.sql(f"create temp stage {stage_name};").collect()
        spec_file = "spec.yaml"
        stage_file = f"@{stage_name}"
        spec = f"{stage_file}/{spec_file}"
        session.file.put_stream(
            BytesIO(
                dedent(f"""\
                    spec:
                        containers:
                        - name: hello-world
                          image: {imagerepo}/hello-world:latest
                    """).encode()
            ),
            spec,
        )

        test_job_spec = JobService(
            name=job_name,
            compute_pool=shared_compute_pool,
            spec=ServiceSpecStageFile(stage=stage_name, spec_file=spec_file),
            is_async_job=True,
        )
        test_job = services.execute_job(test_job_spec)

        fetched_job = test_job.fetch()

        assert fetched_job.name.upper() == job_name.upper()
        assert fetched_job.compute_pool.upper() == shared_compute_pool.upper()
        assert fetched_job.min_instances == 1
        assert fetched_job.max_instances == 1
        assert fetched_job.database_name.upper() == services.database.name.upper()
        assert fetched_job.schema_name.upper() == services.schema.name.upper()
        assert fetched_job.is_job is True
        assert fetched_job.is_async_job is True
    finally:
        if test_job is not None:
            test_job.drop()



from enum import Enum
from typing import Any, Optional


BRIDGE_OVERRIDE_PARAMETER_PREFIX = "ENABLE_SNOW_API_FOR_"


class SnowApiParameter(Enum):
    USE_CLIENT_RETRY = "USE_CLIENT_RETRY"
    PRINT_VERBOSE_STACK_TRACE = "PRINT_VERBOSE_STACK_TRACE"
    FIX_HOSTNAME = "FIX_HOSTNAME"
    MAX_THREADS = "MAX_THREADS"


class SnowApiParameters:
    """Wrapper that abstracts away the behavior from the parsing/reading of parameters.

    Args:
        params_map: A ``dict[str,str]`` of parameter names to their values
    """

    def __init__(self, params_map: dict[SnowApiParameter, Any]) -> None:
        self.params_map = params_map

    def is_parameter_true(self, param_name: SnowApiParameter, default: str) -> bool:
        return (self.params_map.get(param_name, default) or default).lower().strip() in ("true", "t", "yes", "y", "on")

    @property
    def should_retry_request(self) -> bool:
        return self.is_parameter_true(SnowApiParameter.USE_CLIENT_RETRY, "true")

    @property
    def should_print_verbose_stack_trace(self) -> bool:
        return self.is_parameter_true(SnowApiParameter.PRINT_VERBOSE_STACK_TRACE, "true")

    @property
    def fix_hostname(self) -> bool:
        return self.is_parameter_true(SnowApiParameter.FIX_HOSTNAME, "true")

    @property
    def max_threads(self) -> Optional[int]:
        param = self.params_map.get(SnowApiParameter.MAX_THREADS)
        if param is None:
            return None
        return int(param)



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr

from snowflake.core.cortex.search_service._generated.models.scoring_config import ScoringConfig, ScoringConfigModel


class QueryRequest(BaseModel):
    """A model object representing the QueryRequest resource.

    Constructs an object of type QueryRequest with the provided properties.

    Parameters
    __________
    query : str, optional
        Unstructured text query.  Exactly one of 'query' or 'multi_index_query' must be specified.
    multi_index_query : object, optional
        A search query expressed as a collection of multiple column-level queries.  Exactly one of 'query' or 'multi_index_query' must be specified. A column/index can be queried with multiple queries.
    columns : list[str], optional
        List of columns to return.
    filter : object, optional
        Filter query.
    limit : int,  default 10
        Max number of results to return.
    scoring_config : ScoringConfig, optional

    scoring_profile : str, optional
        Mutually exclusive with scoring_config. Specifies the pre-defined name of a profile on the cortex search service that resolves to scoring_config.
    experimental : object, optional
        reserved
    """

    query: Optional[StrictStr] = None

    multi_index_query: Optional[Dict[str, Any]] = None

    columns: Optional[List[StrictStr]] = None

    filter: Optional[Dict[str, Any]] = None

    limit: Optional[StrictInt] = 10

    scoring_config: Optional[ScoringConfig] = None

    scoring_profile: Optional[StrictStr] = None

    experimental: Optional[Dict[str, Any]] = None

    __properties = [
        "query",
        "multi_index_query",
        "columns",
        "filter",
        "limit",
        "scoring_config",
        "scoring_profile",
        "experimental",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> QueryRequest:
        """Create an instance of QueryRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of scoring_config
        if self.scoring_config:
            _dict["scoring_config"] = self.scoring_config.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> QueryRequest:
        """Create an instance of QueryRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return QueryRequest.model_validate(obj)

        _obj = QueryRequest.model_validate(
            {
                "query": obj.get("query"),
                "multi_index_query": obj.get("multi_index_query"),
                "columns": obj.get("columns"),
                "filter": obj.get("filter"),
                "limit": obj.get("limit") if obj.get("limit") is not None else 10,
                "scoring_config": ScoringConfig.from_dict(obj.get("scoring_config"))
                if obj.get("scoring_config") is not None
                else None,
                "scoring_profile": obj.get("scoring_profile"),
                "experimental": obj.get("experimental"),
            }
        )

        return _obj


class QueryRequestModel:
    def __init__(
        self,  # optional properties
        query: Optional[str] = None,
        multi_index_query: Optional[object] = None,
        columns: Optional[list[str]] = None,
        filter: Optional[object] = None,
        limit: Optional[int] = 10,
        scoring_config: Optional[ScoringConfig] = None,
        scoring_profile: Optional[str] = None,
        experimental: Optional[object] = None,
    ):
        """A model object representing the QueryRequest resource.

        Constructs an object of type QueryRequest with the provided properties.

        Parameters
        __________
        query : str, optional
            Unstructured text query.  Exactly one of 'query' or 'multi_index_query' must be specified.
        multi_index_query : object, optional
            A search query expressed as a collection of multiple column-level queries.  Exactly one of 'query' or 'multi_index_query' must be specified. A column/index can be queried with multiple queries.
        columns : list[str], optional
            List of columns to return.
        filter : object, optional
            Filter query.
        limit : int,  default 10
            Max number of results to return.
        scoring_config : ScoringConfig, optional

        scoring_profile : str, optional
            Mutually exclusive with scoring_config. Specifies the pre-defined name of a profile on the cortex search service that resolves to scoring_config.
        experimental : object, optional
            reserved
        """
        self.query = query
        self.multi_index_query = multi_index_query
        self.columns = columns
        self.filter = filter
        self.limit = limit
        self.scoring_config = scoring_config
        self.scoring_profile = scoring_profile
        self.experimental = experimental

    __properties = [
        "query",
        "multi_index_query",
        "columns",
        "filter",
        "limit",
        "scoring_config",
        "scoring_profile",
        "experimental",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return QueryRequest(
            query=self.query,
            multi_index_query=self.multi_index_query,
            columns=self.columns,
            filter=self.filter,
            limit=self.limit,
            scoring_config=self.scoring_config._to_model() if self.scoring_config is not None else None,
            scoring_profile=self.scoring_profile,
            experimental=self.experimental,
        )

    @classmethod
    def _from_model(cls, model) -> QueryRequestModel:
        return QueryRequestModel(
            query=model.query,
            multi_index_query=model.multi_index_query,
            columns=model.columns,
            filter=model.filter,
            limit=model.limit,
            scoring_config=ScoringConfigModel._from_model(model.scoring_config) if model.scoring_config else None,
            scoring_profile=model.scoring_profile,
            experimental=model.experimental,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a QueryRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the QueryRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> QueryRequestModel:
        """Create an instance of QueryRequest from a dict.

        This method constructs a QueryRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        QueryRequest
            A QueryRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(QueryRequest.from_dict(obj))


QueryRequest._model_class = QueryRequestModel
